import datetime

from django.db.models import F
from django.db.models import Q
from rest_framework import serializers
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import *


class ThreadListAPISerializer(serializers.ModelSerializer):
    class Meta:
        model = Thread
        fields = '__all__'


class ThreadListAPIView(APIView):
    permission_classes = (AllowAny,)  # 允许所有用户
    params = None

    print(35 * "-", "api", 35 * "-")

    def get(self, request, format=None):
        self.params = request.query_params  # 返回QueryDict类型

        # 分页
        page = int(self.params['page']) - 1 if 'page' in self.params else 0
        size = int(self.params['size']) if 'size' in self.params else 10

        if 'category_id' not in self.params or self.params['category_id'] == '':
            return Response({'err': 1, 'msg': '缺少参数category_id', 'data': [], 'request': self.params, })

        threads = Thread.objects.all().filter(Q(category_id=self.params['category_id'])).order_by('id')

        if 'classify_id' in self.params:
            threads = Thread.objects.all().filter(
                Q(category_id=self.params['category_id']) &
                Q(classify_id=self.params['classify_id'])
            ).order_by('id')

        if 'title' in self.params:
            threads = Thread.objects.all().filter(
                Q(category_id=self.params['category_id']) &
                Q(title__contains=self.params['title'])
            ).order_by('id')

        if 'content' in self.params:
            threads = Thread.objects.all().filter(
                Q(category_id=self.params['category_id']) &
                Q(content__contains=self.params['content'])
            ).order_by('id')

        if 'classify_id' in self.params and 'title' in self.params:
            threads = Thread.objects.all().filter(
                Q(category_id=self.params['category_id']) &
                Q(classify_id=self.params['classify_id']) &
                Q(title__contains=self.params['title'])
            ).order_by('id')

        if 'classify_id' in self.params and 'content' in self.params:
            threads = Thread.objects.all().filter(
                Q(category_id=self.params['category_id']) &
                Q(classify_id=self.params['classify_id']) &
                Q(content__contains=self.params['content'])
            ).order_by('id')

        if 'title' in self.params and 'content' in self.params:
            threads = Thread.objects.all().filter(
                Q(category_id=self.params['category_id']) &
                Q(title__icontains=self.params['title']) &
                Q(content__icontains=self.params['content'])
            ).order_by('id')

        today = datetime.datetime.now()
        if 'start_time' in self.params:
            start_date = datetime.datetime.strptime(self.params['start_time'], '%Y-%m-%d')
            print(">>>start_date ", start_date)

            if start_date <= today:
                threads = Thread.objects.all().filter(
                    Q(category_id=self.params['category_id']),
                    create_time__range=[start_date, today]
                ).order_by('id')

        if 'end_time' in self.params:
            end_date = datetime.datetime.strptime(self.params['end_time'] + ' 23:59:59', '%Y-%m-%d %H:%M:%S')
            print(">>>end_date ", end_date)

            if end_date >= today:
                threads = Thread.objects.all().filter(
                    Q(category_id=self.params['category_id']),
                    create_time__range=[today, end_date]
                ).order_by('id')

        if 'start_time' in self.params and 'end_time' in self.params:
            start_date = datetime.datetime.strptime(self.params['start_time'], '%Y-%m-%d')
            end_date = datetime.datetime.strptime(self.params['end_time'] + ' 23:59:59', '%Y-%m-%d %H:%M:%S')
            print(">>>start_date ", start_date)
            print(">>>end_date ", end_date)

            if start_date <= today <= end_date:
                threads = Thread.objects.all().filter(
                    Q(category_id=self.params['category_id']),
                    create_time__range=[start_date, end_date]
                ).order_by('id')

        # if 'tags' in self.params:
        #     tags_arrr = self.params['tags'].split(",")
        #
        #     for index in range(len(tags_arrr)):
        #         # 查询是否存在该标签id
        #         threadTags = ThreadTag.objects.all().filter(Q(label=tags_arrr[index])).values('id')
        #         print(">>>", threadTags)



        #         mappings = ThreadTagMapping.objects.update_or_create(tag_id=threadTags,thread=111, id=1)
        #         print(">>>mappings ", mappings)

        #         如果不存在
        #         if not threadTags.exists():
        #             return Response({'err': 1, 'msg': '不存在该标签ID：'+tag_list_arrr[index], 'data': [], 'request': self.params, })
        #         如果存在，则获取eh_thread_tag表的ID，并记录到eh_thread_tag_mapping的tag_id和thread_id
        #         if threadTags.exists():
        #             tag_id = ThreadTag.objects.filter(Q(label))

        # print(">>>sql:")
        # print(threadTags.query)
        # print("\n")

        total = threads.count()
        now_pages = threads[page * size:page * size + size] if page >= 0 else threads
        data = now_pages.annotate(
            thread_category=F('category__label'),
            thread_classify=F('classify__classify'),
            show_type=F('show__label'),
            username=F('user__username'),
            thread_auth=F('auth__label'),
        ).values(
            'id',
            'thread_category',
            'thread_classify',
            'show_type',
            'username',
            'thread_auth',
            'title',
            'content',
            'has_enroll',
            'has_fee',
            'has_comment',
            'cover',
            'photos',
            'video',
            'create_time',
            'update_time',
            'weight',
            'views',
            'plays',
            'comments',
            'likes',
            'favorite',
            'shares',
        )
        return Response({
            'err': 0,
            'msg': 'OK',
            'data': {'total': total, 'list': data, },
            'request': self.params,
        })



# encoding: utf-8
"""
@project: djangoModel->thread_category_apis
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 类别api
@created_time: 2022/10/25 14:40
"""
from rest_framework.views import APIView

from ..services.thread_classify_service import ThreadClassifyService
from ..utils.custom_response import util_response
from ..utils.custom_tool import request_params_wrapper


class ThreadClassifyApis(APIView):
    @request_params_wrapper
    def add(self, *args, request_params=None, **kwargs):
        data, err = ThreadClassifyService.add(request_params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @request_params_wrapper
    def delete(self, *args, request_params=None, **kwargs):
        pk = kwargs.get("pk") or request_params.get("id") or request_params.get("category_id")
        data, err = ThreadClassifyService.delete(pk)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @request_params_wrapper
    def edit(self, *args, request_params=None, **kwargs):
        pk = kwargs.get("pk") or request_params.get("id") or request_params.get("category_id")
        data, err = ThreadClassifyService.edit(request_params, pk)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @request_params_wrapper
    def list(self, *args, request_params=None, **kwargs):
        # print("request_params:", request_params, "kwargs:", kwargs)
        need_pagination = request_params.get("need_pagination")
        filter_fields = request_params.get("filter_fields")
        data, err = ThreadClassifyService.list(filter_fields_params=request_params, need_pagination=need_pagination, filter_fields=filter_fields)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)



"""
Created on 2022-04-11
@author:刘飞
@description:发布子模块逻辑处理
"""
import logging

from django.core.paginator import EmptyPage
from django.core.paginator import PageNotAnInteger
from django.core.paginator import Paginator
from django.db.models import F
from rest_framework import serializers

# from ..models import ThreadAuth
from ..models import ThreadCategory
from ..models import ThreadClassify
from ..models import ThreadExtendField
from ..models import ThreadShow
from ..models import ThreadTag
# from ..serializers import ThreadAuthListSerializer
from ..serializers import ThreadTagSerializer
from ..utils.custom_tool import format_params_handle

log = logging.getLogger()


class ThreadOtherListServices:
    def __init__(self):
        pass

    @staticmethod
    def thread_category(params):
        """
        类别。类似于版块大类的概念，用于圈定信息内容所属的主要类别
        """
        category_set = ThreadCategory.objects.filter(is_deleted=0).order_by("-sort")
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=["id", "platform_code", "category_value", "name", "need_auth", "description", "sort", "parent_id", ],
            alias_dict={"name": "name__contains"}
        )
        if params:
            category_set = category_set.filter(**params)
        thread_category_obj = category_set.annotate(category_value=F('value')).values(
            "id",
            "platform_code",
            "category_value",
            "name",
            "need_auth",
            "description",
            "sort",
            "parent_id",
        )
        if not thread_category_obj:
            return [], None
        return list(thread_category_obj), None

    @staticmethod
    def thread_classify(category_value=None, category_id=None, classify_id=None, classify_value=None, classify_description=None):
        """
        分类。具体的分类，可以是按行业、兴趣、学科的分类，是主类别下的子分类。
        """
        classify_set = ThreadClassify.objects.order_by('sort')
        if category_id:
            classify_set = classify_set.filter(category_id=category_id)
        if category_value:
            classify_set = classify_set.filter(category_id__value=category_value)
        if classify_id:
            classify_set = classify_set.filter(classify_id=classify_id)
        if classify_value:
            classify_set = classify_set.filter(value=classify_value)
        if classify_description:
            classify_set = classify_set.filter(description=classify_description)
        classify_set = classify_set.annotate(category_value=F('category_id__value'))
        classify_set = classify_set.annotate(show_value=F('show_id__value'))
        return list(classify_set.values()), None

    @staticmethod
    def thread_show(params=None, need_pagination=False):
        """
        展示类型。用于对前端界面的显示样式进行分类
        """
        page = params.get("page", 1)
        size = params.get("size", 10)
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "id|int",
                "show_id|int",
                "show_id_list|list",
                "value",
                "value_list|list"
                "name"
            ],
            split_list=["value_list", "name_list", "show_id_list"],
            split_char=params.get("split_char", ";"),
            alias_dict={"show_id": "id", "value_list": "value__in", "show_id_list": "id__in"},
            is_remove_empty=True
        )
        show_query_set = ThreadShow.objects.filter(**params).annotate(label=F("value")).values(
            "id", "value", "name", "label", "config", "description"
        )

        if need_pagination:
            paginator = Paginator(show_query_set, size)
            count = show_query_set.count()
            try:
                show_query_set = paginator.page(page)
            except EmptyPage:
                show_query_set = paginator.page(paginator.num_pages)
            except Exception as e:
                return None, f'{str(e)}'
            return {"page": page, "size": size, "count": count, "list": list(show_query_set.object_list)}, None
        else:
            return list(show_query_set), None

    # @staticmethod
    # def thread_auth(params=None):
    #     """
    #     访问权限。作者指定允许哪里用户可以访问，例如私有、公开、好友、指定某些人可以访问等。
    #     """
    #     thread_auth_obj = ThreadAuth.objects.all()
    #     res = ThreadAuthListSerializer(thread_auth_obj, many=True)
    #     return res.data, None

    @staticmethod
    def thread_tag(params):
        """
        标签类型，存放预置标签。
        """
        size = params.get('size', 10)
        page = params.get('page', 1)
        thread_tag_obj = ThreadTag.objects.all()
        paginator = Paginator(thread_tag_obj, size)
        try:
            thread_tag_obj = paginator.page(page)
        except PageNotAnInteger:
            thread_tag_obj = paginator.page(1)
        except EmptyPage:
            thread_tag_obj = paginator.page(paginator.num_pages)
        except Exception as e:
            log.error(f'信息主表分页:{str(e)}')
            raise serializers.ValidationError(str(e))
        res = ThreadTagSerializer(thread_tag_obj, many=True)
        data = {'total': paginator.count, 'list': res.data}
        return data, None

    @staticmethod
    def thread_extend_field_list(params):
        """获取所有的扩展字段列表"""
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=["category_value", "category_id", "id"],
            alias_dict={"category_value": "category__value"}
        )
        print("params:", params)
        extend_field_obj = ThreadExtendField.objects.annotate(field_label=F("value")).annotate(category_value=F("category__value")).annotate(category_description=F("category__description"))
        extend_field_obj = extend_field_obj.filter(**params)
        obj_list = extend_field_obj.values('category_value', 'category_description', 'category_id', "field", 'field_label', 'type', 'unit', 'config')
        return list(obj_list), None



from django.forms import ModelForm, Textarea, TextInput
from .models import Thread


class ThreadForm(ModelForm):

    class Meta:
        model = Thread
        fields = ('title', )
        widgets = {
            # 'title': Textarea(attrs={'cols': 100, 'rows': 20}),
            'title': TextInput(attrs={'size': 200, 'title': 'Your name'}),
        }



"""
Created on 2022-04-11
@author:刘飞
@description:发布子模块逻辑处理
"""
import logging

from django.core.paginator import EmptyPage
from django.core.paginator import PageNotAnInteger
from django.core.paginator import Paginator
from django.db.models import F
from rest_framework import serializers

from ..models import ThreadAuth
from ..models import ThreadCategory
from ..models import ThreadClassify
from ..models import ThreadExtendField
from ..models import ThreadShow
from ..models import ThreadTag
from ..serializers import ThreadAuthListSerializer
from ..serializers import ThreadTagSerializer
from ..utils.model_handle import format_params_handle

log = logging.getLogger()


class ThreadCategoryTreeServices:
    def __init__(self):
        pass

    @staticmethod
    def get_category_tree(category_id=None, category_value=None):
        """
        类别树。
        """
        # 第一步，把类别列表全部读出来
        category_set = ThreadCategory.objects.annotate(category_value=F('value')).order_by('sort').values(
            'id',
            'platform_code',
            'category_value',
            'name',
            'need_auth',
            'description',
            'sort',
            'parent_id',
        )
        print("> category_set:", category_set)
        category_list = list(category_set)
        print("> category_list:", category_list)

        # 第二步，遍历列表，把数据存放在dict里
        parent_category_dict = {}  # 父类别ID索引列表字典
        category_tree = {}
        for item in category_list:
            # 查找到树根
            if item['id'] == category_id or item['category_value'] == category_value:
                category_tree = item

            # 边界检查：如果没有父类别，不需要生成索引列表字典
            if not item['parent_id']:
                continue
            
            pid = str(item['parent_id'])  # 父ID的字符串

            # 边界检查：如果索引未创建，则初始化
            if not parent_category_dict.get(pid, None):
                parent_category_dict[pid] = []

            # 根据父类别ID插入列表字典
            parent_category_dict[pid].append(item)

        print("> ThreadCategoryTreeServices category_tree:", category_tree)
        print("> ThreadCategoryTreeServices parent_category_dict:", parent_category_dict)

        # 递归树
        def recur_tree(tree_node, parent_dict=None):
            """
            递归树
            @param tree_node 树的每个节点
            @param parent_dict 以每个节点的父ID作为键名的字典
            """
            pid_str = str(tree_node['id'])
            tree_node['children'] = parent_dict.get(pid_str, [])
            for child in tree_node['children']:
                recur_tree(child, parent_dict)
            return tree_node

        if category_tree:
            category_tree = recur_tree(tree_node=category_tree, parent_dict=parent_category_dict)

        if not parent_category_dict:
            return [], None

        return category_tree, None

    @staticmethod
    def get_category_all_tree(category_value=None, category_id=None):
        pass



"""
Created on 2022-04-11
@description:刘飞
@description:发布子模块逻辑分发
"""
import datetime
import re

from rest_framework.views import APIView

from xj_user.services.user_detail_info_service import DetailInfoService
from ..services.thread_category_tree_service import ThreadCategoryTreeServices
from ..services.thread_list_service import ThreadListService
from ..services.thread_statistic_service import StatisticsService
from ..utils.custom_response import util_response
from ..utils.custom_tool import request_params_wrapper, filter_result_field
from ..utils.join_list import JoinList
from ..utils.user_wrapper import user_authentication_wrapper


class ThreadListAPIView(APIView):
    """
    get: 信息表列表
    post: 信息表新增
    """

    # 我们更希望通过装饰器来做权限验证，这样可以更好的精简API层的代码量 2022.10.3 by Sieyoo
    @user_authentication_wrapper  # 如果有token则返回user_info，无则返回空
    @request_params_wrapper
    def get(self, *args, user_info=None, request_params, **kwargs):
        request_params.setdefault("category_value", kwargs.get("category_value", None))

        # 是否检查子树的的信息,如果category_id或者category_value都没传则查询全部
        need_child_category = request_params.pop('need_child_category', None)
        if need_child_category and (request_params.get("category_id") or request_params.get("category_value")):
            category_ids, category_tree_err = ThreadCategoryTreeServices.get_child_ids(
                category_id=request_params.pop("category_id", None),
                category_value=request_params.pop("category_value", None)
            )
            if category_tree_err:
                return util_response(err=1000, msg="获取字数节点错误：" + category_tree_err)
            request_params.setdefault("category_id_list", category_ids)

        # 检查时间格式
        try:
            if request_params.get('create_time_start'):
                datetime.datetime.strptime(request_params.get('create_time_start'), "%Y-%m-%d %H:%M:%S")
            if request_params.get('create_time_end'):
                datetime.datetime.strptime(request_params.get('create_time_end'), "%Y-%m-%d %H:%M:%S")
        except ValueError:
            return util_response(err=1001, msg="时间格式错误:它的格式应该是YYYY-MM-DD HH:MM:SS")

        # 获取列表数据
        thread_serv, error_text = ThreadListService.list(request_params)
        if error_text:
            return util_response(err=1002, msg=error_text)

        # 按权限自动过滤数据
        thread_id_list = list(set([item['id'] for item in thread_serv['list'] if item['id']]))
        user_id_list = list(set([item['user_id'] for item in thread_serv['list'] if item['user_id']]))

        # 用户数据和统计数据 合并
        statistic_list = StatisticsService.statistic_list(id_list=thread_id_list)
        user_info_list = DetailInfoService.get_list_detail(user_id_list=user_id_list)
        thread_serv['list'] = JoinList(l_list=thread_serv['list'], r_list=statistic_list, l_key="id", r_key='thread_id').join()
        thread_serv['list'] = JoinList(l_list=thread_serv['list'], r_list=user_info_list, l_key="user_id", r_key='user_id').join()

        # 是否过滤字段
        filter_fields = request_params.pop('filter_fields', [
            "id", "category_name", "classify_name", "show_value", "title", "subtitle", "summary",
            "cover", "photos", "video", "author", "avatar", "user_name", "nickname",
            "weight", "views", "plays", "comments", "likes", "favorite", "shares", "create_time", "update_time",
        ])
        if isinstance(filter_fields, str):
            # 过滤空格
            filter_fields = re.sub(
                r'\s*;\s*',
                ';',
                filter_fields
            )
            # 拆分成列表
            filter_fields = filter_fields.split(';')

        thread_serv['list'] = filter_result_field(
            result_list=thread_serv['list'],
            filter_filed_list=filter_fields
        )

        return util_response(data=thread_serv, is_need_parse_json=True)



# encoding: utf-8
"""
@project: djangoModel->validator
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 验证器
@created_time: 2022/7/29 11:44
"""
from .models import ThreadClassify, ThreadCategory
from .utils.validator import *


def is_set_classify_id(value):
    res = list(ThreadClassify.objects.filter(id=value).values())
    if not res:
        raise ValidationError('无法匹配到classify_id，请先创建')


def is_set_category_id(value):
    res = list(ThreadCategory.objects.filter(id=value).values())
    if not res:
        raise ValidationError('无法匹配到category_id，请先创建')


class ThreadAddValidator(Validate):
    # category_id = forms.IntegerField(
    #     required=True,
    #     error_messages={
    #         "required": "category_id 必填",
    #     },
    #     validators=[is_set_category_id]
    # )
    #
    classify_id = forms.IntegerField(
        required=True,
        error_messages={
            "required": "classify_id 必填",
        },
        validators=[is_set_classify_id]
    )

    title = forms.CharField(
        required=True,
        error_messages={
            "required": "title 必填,请先设置然后进行绑定",
        })

    content = forms.CharField(
        required=True,
        error_messages={
            "required": "content 必填,请先设置然后进行绑定",
        })


class ThreadUpdateValidator(Validate):
    id = forms.IntegerField(
        required=True,
        error_messages={
            "required": "id 必填",
        }
    )



"""
Created on 2022-04-11
@description:刘飞
@description:发布子模块逻辑分发
"""
from rest_framework.response import Response
from rest_framework.views import APIView

from ..services.thread_category_tree_service import ThreadCategoryTreeServices
from ..utils.custom_response import util_response
from ..utils.custom_tool import parse_data
from ..utils.user_wrapper import user_authentication_force_wrapper


class ThreadCategoryTreeAPIView(APIView):
    """
    get: 信息表列表
    post: 信息表新增
    """

    def get(self, request, category_value=None, *args, **kwargs):
        params = parse_data(request)
        print("> ThreadCategoryTreeAPIView params category_value:", params, category_value)
        category_value = category_value if category_value else params.get('category_value', None)
        category_id = params.get('category_id', None)
        print("> ThreadCategoryTreeAPIView category_value, category_id:", category_value, category_id)
        if category_value or category_id:
            category_serv, error_text = ThreadCategoryTreeServices.get_category_tree(category_id=category_id, category_value=category_value)
        else:
            category_serv, error_text = ThreadCategoryTreeServices.get_category_tree()

        if error_text:
            return Response({'err': 1000, 'msg': error_text})

        return Response({'err': 0, 'msg': 'OK', 'data': category_serv})

    # V2版本列别树
    @user_authentication_force_wrapper
    def get_category_tree_by_user(self, *args, user_info=None, **kwargs):
        if user_info is None:
            user_info = {}
        user_id = user_info.get("user_id")
        category_serv, error_text = ThreadCategoryTreeServices.get_category_tree_by_user(user_id=user_id)
        if error_text:
            return util_response(err=1000, msg=error_text)
        return util_response(data=category_serv)



# encoding: utf-8
"""
@project: djangoModel->thread_category_item_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 类别单条服务
@created_time: 2022/10/25 14:21
"""
from django.core.paginator import Paginator
from django.db.models import F

from ..models import ThreadCategory
from ..utils.custom_tool import format_params_handle, format_list_handle


class ThreadCategoryService():
    @staticmethod
    def edit(edit_params=None, pk=None, search_params=None):
        filtered_add_params = format_params_handle(
            param_dict=edit_params or {},
            filter_filed_list=["platform_code", "value", "name", "need_auth", "description", "sort", "parent_id", ]
        )
        # print("edit_params:", filtered_add_params, "pk:", pk, "search_params:", search_params)
        if not pk and not search_params:
            return None, "没有可编辑的数据"
        # 搜索可修改的数据
        category_obj = ThreadCategory.objects
        if pk:
            category_obj = category_obj.filter(id=pk)
        if search_params:
            category_obj = category_obj.filter(**search_params)

        if not category_obj:
            return None, "没找到可修改的数据"

        instance = category_obj.update(**edit_params)
        return instance, None

    @staticmethod
    def delete(pk=None, search_params=None):
        if not pk and not search_params:
            return None, "没有可删除的数据"
        # 搜索可修改的数据
        category_obj = ThreadCategory.objects
        if pk:
            category_obj = category_obj.filter(id=pk)
        if search_params:
            category_obj = category_obj.filter(**search_params)

        if not category_obj:
            return None, "没找到可删除的数据"
        category_obj.update(is_deleted=1)
        return None, None

    @staticmethod
    def add(add_params):
        if not add_params:
            return None, "参数不能为空"

        add_params['parent_id'] = None if not add_params.get("parent_id") else add_params.get("parent_id")
        add_params['platform_code'] = None if not add_params.get("platform_code") else add_params.get("platform_code")
        filtered_add_params = format_params_handle(
            param_dict=add_params,
            filter_filed_list=["id", "platform_code", "value", "name", "need_auth", "description", "sort", "parent_id", ],
            is_remove_null=True
        )
        try:
            filtered_add_params['is_deleted'] = False
            instance = ThreadCategory.objects.create(**filtered_add_params)
        except Exception as e:
            return None, str(e)
        return instance.to_json(), None

    @staticmethod
    def list(filter_fields_params=None, filter_fields=None, need_pagination=True):
        """
        类别。类似于版块大类的概念，用于圈定信息内容所属的主要类别
         """
        # 参数筛选
        if filter_fields_params is None:
            filter_fields_params = {}

        page = filter_fields_params.pop("page", 1)
        size = filter_fields_params.pop("size", 10)

        # 查询参数过滤，替换
        filter_fields_params = format_params_handle(
            param_dict=filter_fields_params,
            filter_filed_list=["id", "platform_code", "value", "category_value", "name", "need_auth", "description", "sort", "parent_id", "parent_value"],
            alias_dict={"name": "name__contains", "value": "category_value"}
        )

        # 查询字段筛选
        default_fields = ["id", "platform_code", "category_value", "name", "need_auth", "description", "sort", "parent_value", "parent_id", "is_deleted"]
        filter_fields_list = format_list_handle(
            param_list=filter_fields.split(";") if filter_fields else [],
            filter_filed_list=default_fields
        )
        filter_fields_list = filter_fields_list or default_fields

        # 数据库orm查询
        category_set = ThreadCategory.objects.filter(is_deleted=0).annotate(
            category_value=F('value'),
            parent_value=F("parent__value")
        ).order_by("sort")
        category_set = category_set.filter(**filter_fields_params)
        thread_category_obj = category_set.values(*filter_fields_list)

        # 不需要分页展示全部数据
        if not need_pagination:
            if not thread_category_obj:
                return [], None
            return list(thread_category_obj), None

        # 分页展示
        count = thread_category_obj.count()
        finish_set = list(Paginator(thread_category_obj, size).page(page))
        return {'size': int(size), 'page': int(page), 'total': count, 'list': finish_set}, None



# encoding: utf-8
"""
@project: djangoModel->thread_v2
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis:
@created_time: 2022/7/29 15:11
"""
from django.core.paginator import Paginator
from django.db import transaction

from ..models import Thread
from ..models import ThreadStatistic


# 统计服务
class StatisticsService:
    @staticmethod
    def increment(thread_id, tag, step=1):
        """
        增量计数
        :param thread_id:
        :param tag: 递增的字段
        :param step:  递增的步长
        :return: （err, data） 或者 util_response({'err': err, 'msg': msg, 'data': data})
        """
        query_obj = ThreadStatistic.objects.filter(thread_id=thread_id)
        match_data = query_obj.first()
        if match_data:
            form = {tag + "": getattr(match_data, tag) + int(step)}
            query_obj.update(**form)
        else:
            form = {"thread_id": thread_id, tag: step}
            ThreadStatistic.objects.create(**form)
        return None, 0

    @staticmethod
    def increments(thread_id, increment_dict):
        """
        批量计数增量统计
        :param thread_id:  关联主键
        :param increment_dict: {递增字段：递增的值}
        :return:（err, data） 或者 util_response({'err': err, 'msg': msg, 'data': data})
        """
        is_set_thread = Thread.objects.filter(id=thread_id)
        if not is_set_thread:
            return None, '不存该条信息'
        # 事务回滚
        sid = transaction.savepoint()
        try:
            for k, v in increment_dict.items():
                StatisticsService.increment(thread_id, k, v)
            return None, 0
        except Exception as e:
            transaction.savepoint_rollback(sid)
            return None, "写入异常" + str(e)

    @staticmethod
    def statistic_list(params=None, id_list=None):
        if id_list is None:
            page = params.pop("page", 1)
            size = params.pop("size", 20)
            res_set = ThreadStatistic.objects.filter(**params)
            count = res_set.count()
            res_set = list(Paginator(res_set.values(), size).page(page))
            return {'size': int(size), 'page': int(page), 'count': count, 'list': res_set}, None
        else:
            return list(ThreadStatistic.objects.filter(thread_id__in=id_list).values())



# encoding: utf-8
"""
@project: djangoModel->thread_category_item_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 类别单条服务
@created_time: 2022/10/25 14:21
"""
from django.core.paginator import Paginator
from django.db.models import F

from ..models import ThreadClassify
from ..utils.custom_tool import format_params_handle, format_list_handle


class ThreadClassifyService():
    @staticmethod
    def edit(edit_params=None, pk=None, search_params=None):
        filtered_edit_params = format_params_handle(
            param_dict=edit_params or {},
            filter_filed_list=["value", "name", "show", "show_id", "description", "category", "category_id", "icon", "sort", "parent_id", "config", ],
        )
        # print("edit_params:", filtered_add_params, "pk:", pk, "search_params:", search_params)
        if not pk and not search_params:
            return None, "没有可编辑的数据"
        # 搜索可修改的数据
        classify_obj = ThreadClassify.objects
        if pk:
            classify_obj = classify_obj.filter(id=pk)
        if search_params:
            classify_obj = classify_obj.filter(**search_params)

        if not classify_obj:
            return None, "没找到可修改的数据"

        instance = classify_obj.update(**filtered_edit_params)
        return instance, None

    @staticmethod
    def delete(pk=None, search_params=None):
        if not pk and not search_params:
            return None, "没有可删除的数据"
        # 搜索可修改的数据
        classify_obj = ThreadClassify.objects
        if pk:
            classify_obj = classify_obj.filter(id=pk)
        if search_params:
            classify_obj = classify_obj.filter(**search_params)

        if not classify_obj:
            return None, "没找到可删除的数据"
        classify_obj.delete()
        return None, None

    @staticmethod
    def add(add_params):
        if not add_params:
            return None, "参数不能为空"

        add_params['parent_id'] = None if not add_params.get("parent_id") else add_params.get("parent_id")
        filtered_add_params = format_params_handle(
            param_dict=add_params,
            filter_filed_list=["id", "value", "name", "show", "show_id", "description", "category", "category_id", "icon", "sort", "parent_id", "config", ],
            is_remove_null=True
        )
        try:
            instance = ThreadClassify.objects.create(**filtered_add_params)
        except Exception as e:
            return None, str(e)
        return instance.to_json(), None

    @staticmethod
    def list(filter_fields_params=None, filter_fields=None, need_pagination=True):
        """
        类别。类似于版块大类的概念，用于圈定信息内容所属的主要类别
         """
        # 参数筛选
        if filter_fields_params is None:
            filter_fields_params = {}
        # 分页参数
        page = filter_fields_params.pop("page", 1)
        size = filter_fields_params.pop("size", 10)

        # 搜索参数过滤
        filter_fields_params = format_params_handle(
            param_dict=filter_fields_params,
            filter_filed_list=[
                "id", "value", "name", "sort",
                "show_id", "show_value", "category_id", "category_value", "parent_id", "parent_value"
            ],
            alias_dict={"name": "name__contains"}
        )

        # 查询字段筛选
        default_fields = [
            "id", "value", "name", "description", "show_id", "show_value", "category_id", "category_value",
            "icon", "sort", "parent_id", "parent_value", "config"
        ]
        filter_fields_list = format_list_handle(
            param_list=filter_fields.split(";") if filter_fields else [],
            filter_filed_list=default_fields
        )

        classify_set = ThreadClassify.objects.annotate(
            category_value=F('category__value'),
            parent_value=F("parent__value"),
            show_value=F("show__value")
        ).order_by("sort")
        classify_set = classify_set.filter(**filter_fields_params)
        thread_classify_obj = classify_set.values(*filter_fields_list)

        # 不需要分页展示全部数据
        if not need_pagination:
            if not thread_classify_obj:
                return [], None
            return list(thread_classify_obj), None

        # 分页展示
        count = thread_classify_obj.count()
        finish_set = list(Paginator(thread_classify_obj, size).page(page))
        return {'size': int(size), 'page': int(page), 'total': count, 'list': finish_set}, None



# encoding: utf-8
"""
@project: djangoModel->thread_static
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 统计接口
@created_time: 2022/8/8 10:05
"""

# 计数统计
from rest_framework.views import APIView

from ..services.thread_statistic_service import StatisticsService
from ..utils.custom_response import util_response
from ..utils.custom_tool import parse_data


class ThreadStaticAPIView(APIView):

    # 统计列表
    def get(self, request):
        params = parse_data(request)
        data, err_txt = StatisticsService.statistic_list(params)
        if not err_txt:
            return util_response(data=data)
        return util_response(err=47767, msg=err_txt)

    # 单条自增
    def post(self, request):
        thread_id = request.data.get('thread_id', None)
        tag = request.data.get('tag', None)
        step = request.data.get('step', None)
        tag_list = ['step', 'views', 'plays', 'comments', 'likes', 'favorite', 'shares']
        if not thread_id or not tag in tag_list:
            return util_response(err=45767, msg='参数错误')
        data, err_txt = StatisticsService.increment(thread_id, tag, step)
        if err_txt == 0:
            return util_response()
        return util_response(err=47767, msg=err_txt)

    # 多字段自增
    def put(self, request):
        form_data = parse_data(request)
        thread_id = form_data.pop('thread_id', None)
        if thread_id is None:
            return util_response(msg="参数错误", err=57766)
        data, err_txt = StatisticsService.increments(thread_id, form_data)
        if err_txt == 0:
            return util_response(data=data)
        return util_response(err=47767, msg=err_txt)



"""
Created on 2022-04-11
@description:刘飞
@description:发布子模块单挑数据删除/修改/详情
"""
from rest_framework.views import APIView

from xj_enroll.service.enroll_services import EnrollServices
from ..services.thread_item_service import ThreadItemService
from ..utils.custom_authentication_wrapper import authentication_wrapper
from ..utils.custom_response import util_response
from ..utils.custom_tool import parse_data

item_service = ThreadItemService()


class ThreadItemAPI(APIView):
    """单挑信息处理，查，改，删"""

    def get(self, request, pk, *args, **kwargs):
        """信息表详情"""
        if not pk:
            return util_response(msg="非法请求", err=2554)
        data, error_text = item_service.detail(pk)

        if not error_text:
            return util_response(data=data)
        return util_response(err=47767, msg=error_text)

    @authentication_wrapper
    def put(self, request, pk, *args, **kwargs):
        """信息表编辑"""
        if not pk:
            return util_response(msg="非法请求", err=1000)
        form_data = parse_data(request=request)
        data, error_text = item_service.edit(form_data, pk)

        # 报名表同步修改category_id
        category_id = form_data.get("category_id", None)
        if category_id:
            data, err = EnrollServices.enroll_edit({"category_id": category_id}, enroll_id=None, search_param={"thread_id": pk})
            if err:
                print("err:", err)
            #     return util_response(err=1001, msg=err)

        if not error_text:
            return util_response()
        return util_response(err=1002, msg=error_text)

    @authentication_wrapper
    def delete(self, request, pk, *args, **kwargs):
        data, error_text = item_service.delete(pk)
        print(error_text)
        if not error_text:
            return util_response()
        return util_response(err=47767, msg=error_text)



from django.contrib import admin

from config.config import JConfig
from .models import Thread, ThreadStatistic
from .models import ThreadExtendData
from .models import ThreadExtendField
from .models import ThreadImageAuth
from .models import ThreadShow, ThreadClassify, ThreadCategory
from .models import ThreadTag, ThreadTagMapping

config = JConfig()


# @admin.register(ThreadAuth)
# class ThreadAuthAdmin(admin.ModelAdmin):
#     list_display = ('id', 'value')
#     search_fields = ('id', 'value')
#     fields = ('id', 'value',)
#     readonly_fields = ('id',)

@admin.register(Thread)
class ThreadAdmin(admin.ModelAdmin):
    list_display = (
        'id', 'is_deleted', 'category', 'classify', 'show', 'user_id', 'with_user_id', 'short_title',
        'short_subtitle', 'short_summary', 'short_content', 'access_level', 'author', 'ip', 'has_enroll', 'has_fee', 'has_comment',
        'has_location', 'short_cover', 'short_photos', 'short_video', 'short_files', 'price', 'is_original', 'link', 'create_time',
        'update_time', 'short_logs', 'short_more', 'sort', 'language_code',
    )
    search_fields = (
        'title', 'subtitle', 'summary', 'content',
    )
    fields = (
        'id', 'is_deleted', 'category', 'classify', 'show', 'user_id', 'with_user_id', 'title',
        'subtitle', 'summary', 'content', 'access_level', 'author', 'ip', 'has_enroll', 'has_fee', 'has_comment',
        'has_location', 'cover', 'photos', 'video', 'files', 'price', 'is_original', 'link', 'create_time',
        'update_time', 'logs', 'more', 'sort', 'language_code',
    )
    readonly_fields = ('id', 'update_time',)  # 只读
    list_per_page = 10  # 每页显示10条
    ordering = ['-update_time']  # 排序

    # class Media:
    #     css = {
    #         'all': (
    #             '/css/fancy.css',
    #         )
    #     }


@admin.register(ThreadCategory)
class ThreadCategoryAdmin(admin.ModelAdmin):
    list_display = ('id', 'platform_code', 'value', 'name', 'need_auth', 'short_description', 'sort', 'parent', "is_deleted")
    search_fields = ('id', 'platform_code', 'value', 'name',)
    fields = ('id', 'platform_code', 'value', 'name', 'need_auth', 'description', 'sort', 'parent', "is_deleted")
    readonly_fields = ('id',)
    ordering = ['platform_code', 'parent', 'sort']


@admin.register(ThreadClassify)
class ThreadClassifyAdmin(admin.ModelAdmin):
    list_display = ('id', 'value', 'name', 'show', 'category', 'description', 'icon', 'sort', 'parent', "config")
    search_fields = ('id', 'value', 'name', 'show', 'category')
    fields = ('id', 'value', 'name', 'show', 'category', 'description', 'icon', 'sort', 'parent', "config")
    readonly_fields = ['id']
    ordering = ['category', 'parent', 'sort']


@admin.register(ThreadShow)
class ThreadShowAdmin(admin.ModelAdmin):
    list_display = ('id', 'value', 'name', 'config', 'description')
    search_fields = ('id', 'value', 'name', 'config')
    fields = ('id', 'value', 'config', 'name', 'description')
    readonly_fields = ('id',)


@admin.register(ThreadExtendData)
class ThreadExtendDataAdmin(admin.ModelAdmin):
    list_display = (
        'thread_id', 'short_field_1', 'short_field_2', 'short_field_3', 'short_field_4', 'short_field_5',
        'short_field_6',
        'short_field_7',
        'short_field_8', 'short_field_9', 'short_field_10', 'short_field_11', 'short_field_12', 'short_field_13',
        'short_field_14',
        'short_field_15', 'short_field_16', 'short_field_17', 'short_field_18', 'short_field_19', 'short_field_20')
    search_fields = (
        'thread_id', 'short_field_1', 'short_field_2', 'short_field_3', 'short_field_4', 'short_field_5',
        'short_field_6',
        'short_field_7',
        'short_field_8', 'short_field_9', 'short_field_10', 'short_field_11', 'short_field_12', 'short_field_13',
        'short_field_14',
        'short_field_15', 'short_field_16', 'short_field_17', 'short_field_18', 'short_field_19', 'short_field_20')
    fields = ('thread_id', 'field_1', 'field_2', 'field_3', 'field_4', 'field_5', 'field_6', 'field_7',
              'field_8', 'field_9', 'field_10', 'field_11', 'field_12', 'field_13', 'field_14',
              'field_15', 'field_16', 'field_17', 'field_18', 'field_19', 'field_20')


@admin.register(ThreadExtendField)
class ThreadExtendFieldAdmin(admin.ModelAdmin):
    list_display = ("id", "category", 'field', 'field_index', 'value', 'type', 'unit')
    search_fields = ("category__value", 'field_index', 'type')
    fields = ('id', "category", 'field', 'field_index', 'value', 'type', 'unit', 'config')
    readonly_fields = ('id',)
    ordering = ['-category_id']
    list_per_page = 20  # 每页显示20条


@admin.register(ThreadTag)
class ThreadTagAdmin(admin.ModelAdmin):
    list_display = ('id', 'value')
    search_fields = ('id', 'value')
    fields = ('id', 'value',)
    readonly_fields = ('id',)


@admin.register(ThreadTagMapping)
class ThreadTagMappingAdmin(admin.ModelAdmin):
    list_display = ('id', 'thread_id', 'tag_id')
    search_fields = ('id', 'thread_id', 'tag_id')
    fields = ('thread_id', 'tag_id')


@admin.register(ThreadImageAuth)
class ThreadImageAuthAdmin(admin.ModelAdmin):
    list_display = ('id', 'value')
    search_fields = ('id', 'value')
    fields = ('id', 'value',)
    readonly_fields = ('id',)


# @admin.register(ThreadResource)
# class ThreadImageAdmin(admin.ModelAdmin):
#     list_display = (
#         'id', 'name', 'url', 'filename', 'filetype', 'image_auth_id', 'price', 'snapshot', 'format', 'logs', 'user_id')
#     search_fields = (
#         'id', 'name', 'url', 'filename', 'filetype', 'image_auth_id', 'price', 'snapshot', 'format', 'logs', 'user_id')
#     fields = (
#         'id', 'name', 'url', 'filename', 'filetype', 'image_auth_id', 'price', 'snapshot', 'format', 'logs', 'user_id')
#     readonly_fields = ('id',)


@admin.register(ThreadStatistic)
class ThreadStatisticAdmin(admin.ModelAdmin):
    list_display = (
        'thread_id', 'flag_classifies', 'flag_weights', 'weight', 'views', 'plays', 'comments', 'likes',
        'favorite', 'shares',
    )
    search_fields = (
        'thread_id', 'flag_classifies', 'flag_weights', 'weight', 'views', 'plays', 'comments', 'likes',
        'favorite', 'shares',
    )
    fields = (
        'thread_id', 'flag_classifies', 'flag_weights', 'weight', 'views', 'plays', 'comments', 'likes',
        'favorite', 'shares',
    )


admin.site.site_header = config.get('main', 'app_name', 'msa一体化管理后台')
admin.site.site_title = config.get('main', 'app_name', 'msa一体化管理后台')



"""
Created on 2022-04-11
@author:刘飞
@description:发布子模块序列化器
"""

from django.utils.translation import gettext as tr
from rest_framework import serializers

from xj_user.models import BaseInfo, DetailInfo
from .models import (ThreadShow,
                     ThreadClassify,
                     ThreadCategory,
                     Thread,
                     ThreadStatistic,
                     ThreadTag,
                     ThreadTagMapping, ThreadExtendField)


# 捕获异常装饰器
def catch_except(func):
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception:
            return ""

    return wrapper


class ThreadShowListSerializer(serializers.ModelSerializer):
    """展示类型序列化器"""
    label = serializers.SerializerMethodField()

    class Meta:
        model = ThreadShow
        fields = '__all__'

    def get_label(self, instance):
        return tr(instance.value)


class ThreadClassifyListSerializer(serializers.ModelSerializer):
    """分类序列化器"""
    label = serializers.SerializerMethodField()

    class Meta:
        model = ThreadClassify
        fields = '__all__'

    def get_label(self, instance):
        return tr(instance.value)


class ThreadCategoryListSerializer(serializers.ModelSerializer):
    """类别序列化"""
    label = serializers.SerializerMethodField()

    class Meta:
        model = ThreadCategory
        fields = ("id", "label", "description", "nead_auth")

    def get_label(self, instance):
        return str(getattr(instance, "value", ""))


# class ThreadAuthListSerializer(serializers.ModelSerializer):
#     """访问权限序列化"""
#     label = serializers.SerializerMethodField()
#
#     class Meta:
#         model = ThreadAuth
#         fields = ("id", "label")
#
#     def get_label(self, instance):
#         return tr(instance.value)


class ThreadStatisticListSerializer(serializers.ModelSerializer):
    """信息统计表序列化器"""

    class Meta:
        model = ThreadStatistic
        fields = [
            # "thread_id",
            # "flag_classifies",
            # "flag_weights",
            "weight",
            "views",
            "plays",
            "comments",
            "likes",
            "favorite",
            "shares",
        ]


class ThreadTagSerializer(serializers.ModelSerializer):
    """标签类型序列化器"""

    class Meta:
        model = ThreadTag
        exclude = ['thread']


class ThreadListSerializer(serializers.ModelSerializer):
    """信息表，即主表序列化"""

    create_time = serializers.DateTimeField(format='%Y-%m-%d %H:%M:%S', read_only=True)
    update_time = serializers.DateTimeField(format='%Y-%m-%d %H:%M:%S', read_only=True)
    # category_value = serializers.SerializerMethodField()
    # classify_value = serializers.SerializerMethodField()
    # show_value = serializers.SerializerMethodField()
    # auth_value = serializers.SerializerMethodField()
    # classify_label = serializers.SerializerMethodField()
    # auth_label = serializers.SerializerMethodField()
    # full_name = serializers.SerializerMethodField()  # 由于耦合了外部模块的用户表格，禁止使用
    # avatar = serializers.SerializerMethodField()  # 由于耦合了外部模块的用户表格，禁止使用
    # statistic = serializers.SerializerMethodField()
    # tag_list = serializers.SerializerMethodField()
    thread_extends = serializers.SerializerMethodField()

    class Meta:
        model = Thread
        fields = [
            # ----- Thread 信息主表 -----
            'id',
            'category_id',
            'category_value',
            'classify_id',
            'classify_value',
            'show_id',
            'show_value',
            'user_id',
            # 'avatar',  # 由于耦合了外部模块的用户表格，禁止使用
            # 'full_name',  # 由于耦合了外部模块的用户表格，禁止使用
            'auth_id',
            # 'auth_value',
            # 'auth_label',
            # 'is_deleted',
            'title',
            'summary',
            # 'content',
            'author',
            # 'ip',
            'has_enroll',
            'has_fee',
            'has_comment',
            'cover',
            'photos',
            'video',
            'files',
            'price',
            'is_original',
            'more',
            'create_time',
            'update_time',

            # ----- Thread Tags 标签信息表 -----
            # 'tag_list',

            # ----- Thread Statistic 统计信息表 -----
            # 'statistic',

            # ----- Thread Extend 扩展信息表 -----
            'thread_extends',
        ]
        # exclude = ('logs',)
        # fields = '__all__'

    def get_show_value(self, instance):
        show_value = instance.show_id.value if instance.show_id else None
        return show_value

    # def get_auth_value(self, instance):
    #     auth_value = instance.auth_id.value if instance.auth_id else None
    #     return auth_value

    def get_classify_label(self, instance):
        classify_value = tr(instance.classify_id.value) if instance.classify_id else None
        return classify_value

    # def get_auth_label(self, instance):
    #     auth_value = tr(instance.auth_id.value) if instance.auth_id else None
    #     return auth_value

    def get_thread_extends(self, instance):
        if not hasattr(instance, 'thread_extend_data'):
            return {}
        fields = ThreadExtendField.objects.filter(classify_id=instance.classify_id)
        return {field.field: getattr(instance.thread_extend_data, field.extend_field) for field in fields}


class ThreadDetailSerializer(serializers.ModelSerializer):
    """信息表，即主表详情序列化"""

    create_time = serializers.DateTimeField(format='%Y-%m-%d %H:%M:%S', read_only=True)
    update_time = serializers.DateTimeField(format='%Y-%m-%d %H:%M:%S', read_only=True)
    category_value = serializers.SerializerMethodField()
    classify_value = serializers.SerializerMethodField()
    show_value = serializers.SerializerMethodField()
    # auth_value = serializers.SerializerMethodField()
    show_label = serializers.SerializerMethodField()
    # auth_label = serializers.SerializerMethodField()
    full_name = serializers.SerializerMethodField()
    user_name = serializers.SerializerMethodField()
    nickname = serializers.SerializerMethodField()
    statistic = serializers.SerializerMethodField()
    tag_list = serializers.SerializerMethodField()
    avatar = serializers.SerializerMethodField()
    thread_extends = serializers.SerializerMethodField()

    class Meta:
        model = Thread
        fields = '__all__'

    @catch_except
    def get_avatar(self, instance):
        user_detail_obj = DetailInfo.objects.filter(user_id=instance.user_id).first()
        avatar = user_detail_obj.avatar if user_detail_obj else None
        return avatar

    @catch_except
    def get_category_value(self, instance):
        category_value = instance.category_id.value if instance.category_id else None
        return category_value

    @catch_except
    def get_classify_value(self, instance):
        return instance.classify_id.value

    @catch_except
    def get_show_value(self, instance):
        show_value = instance.show.value if instance.show_id else None
        return show_value

    # @catch_except
    # def get_auth_value(self, instance):
    #     auth_value = instance.auth.value if instance.auth_id else None
    #     return auth_value

    @catch_except
    def get_show_label(self, instance):
        show_value = tr(instance.show.value) if instance.show_id else None
        return show_value

    # # @catch_except
    # def get_auth_label(self, instance):
    #     auth_value = tr(instance.auth_id.value) if instance.auth_id else None
    #     return auth_value

    @catch_except
    def get_full_name(self, instance):
        user_obj = BaseInfo.objects.filter(id=instance.user_id).first()
        full_name = user_obj.full_name if user_obj else None
        return full_name

    @catch_except
    def get_user_name(self, instance):
        user_obj = BaseInfo.objects.filter(id=instance.user_id).first()
        user_name = user_obj.user_name if user_obj else None
        return user_name

    def get_nickname(self, instance):
        nickname = ""
        if instance.user_id:
            user_obj = BaseInfo.objects.filter(id=instance.user_id).first()
            nickname = user_obj.nickname if user_obj else None
        return nickname

    def get_statistic(self, instance):
        obj = ThreadStatistic.objects.filter(thread_id=instance.id).first()
        return ThreadStatisticListSerializer(obj).data

    def get_tag_list(self, instance):
        tag_list = []
        tag_id_list = ThreadTagMapping.objects.filter(thread_id=instance).values_list('tag_id', flat=True)
        tag_obj = ThreadTag.objects.filter(id__in=tag_id_list)
        for t in tag_obj:
            tag_list.append({"id": t.id, "value": t.value})
        return tag_list

    def get_thread_extends(self, instance):
        if not hasattr(instance, 'thread_extend_data'):
            return {}
        print("instance.category:",instance)
        fields = ThreadExtendField.objects.filter(category_id=instance.category.id)
        return {field.field: getattr(instance.thread_extend_data, field.field_index, None) for field in fields}



# encoding: utf-8
"""
@project: djangoModel->thread_v2
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis:
@created_time: 2022/7/29 15:11
"""
import logging

from django.core.paginator import Paginator
from django.db.models import F

from xj_thread.services.thread_extend_service import ThreadExtendOutPutService
from ..models import Thread, ThreadTagMapping
from ..utils.custom_tool import filter_result_field, format_params_handle

log = logging.getLogger()


# 信息服务CURD(支持扩展字段配置)
class ThreadListService:
    @staticmethod
    def list(params):
        """
        信息列表
        @param params 筛选条件
        @param filter_fields 过渡条件
        """
        # ================== 参数处理 start ==================
        page = params.pop('page', 1)
        size = params.pop('size', 10)
        sort = params.pop('sort', None)
        sort = sort if sort and sort in ['id', '-id', 'sort', '-sort', 'create_time', '-create_time', 'update_time', '-update_time'] else None
        if int(size) > 100:
            size = 10
        exclude_category_list = params.pop('exclude_category_list').split(',') if params.get('exclude_category_list') else None

        # 标签搜索
        tag_id_list = params.get('tag_id_list') if params.get('tag_id_list') else None
        if tag_id_list:
            try:
                id_list = params.pop("id_list", None)
                if not id_list or not isinstance(id_list, list):
                    id_list = []
                params["id_list"] = list(set(id_list + ThreadTagMapping.objects.filter(tag_id__in=tag_id_list).values_list('thread_id', flat=True)))
            except ValueError as e:
                log.error(f'信息表标签查询{e}')

        # 允许进行过渡的字段条件
        conditions = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "category_id|int", "category_name", "category_value", "category_id_list|list", "category_parent_id|int", "platform_code",
                "classify_id|int", "classify_name", "classify_value", "classify_id_list|list", "classify_parent_id|int", "show_value",
                "user_id|int", "user_id_list|list", "id_list|list",
                "title", "create_time_start|date", "create_time_end|date", "access_level", "has_enroll", "has_fee", "has_comment", "need_auth"
            ],
            alias_dict={
                "id_list": "id__in",
                "user_id_list": "user_id__in",
                "category_id_list": "category_id__in",
                "category_value": "category__value",
                "category_parent_id": "category__parent_id",
                "platform_code": "category__platform_code",
                "classify_value": "classify__value",
                "classify_id_list": "classify__in",
                "classify_parent_id": "classify__parent_id",
                "title": "title__contains",
                "create_time_start": "create_time__gte",
                "create_time_end": "create_time__lte",
            },
            split_list=["id_list", "category_id_list", "classify_id_list", "user_id_list"],
            is_remove_empty=True,
        )
        # ================== 参数处理 end ==================

        # ==================== 数据检索 start ====================
        thread_set = Thread.objects
        # 排序
        if sort:
            thread_set = thread_set.order_by(sort)
        # 指定不需要过滤的类别字段
        if exclude_category_list:
            thread_set = thread_set.exclude(category_id__in=exclude_category_list)
        # 开始按过滤条件
        try:
            thread_set = thread_set.annotate(category_value=F("category_id__value")) \
                .annotate(category_name=F("category_id__name")) \
                .annotate(need_auth=F("category_id__need_auth")) \
                .annotate(classify_value=F("classify_id__value")) \
                .annotate(classify_name=F("classify_id__name")) \
                .annotate(show_value=F("show_id__value"))
            thread_set = thread_set.filter(is_deleted=False).filter(**conditions)
            count = thread_set.count()
            thread_set = thread_set.values()
        except Exception as e:
            return None, "err:" + e.__str__() + "line:" + str(e.__traceback__.tb_lineno)
        # 分页数据
        finish_set = list(Paginator(thread_set, size).page(page).object_list)
        # ==================== 数据检索 end ====================

        # ================= 扩展数据拼接  start=================
        category_id_list = list(set([item['category_id'] for item in finish_set if item['category_id']]))
        thread_id_list = list(set([item['id'] for item in finish_set if item['id']]))
        extend_merge_service = ThreadExtendOutPutService(category_id_list=category_id_list, thread_id_list=thread_id_list)
        finish_list = extend_merge_service.merge(finish_set)
        # ================= 扩展数据拼接  end  =================

        return {'size': int(size), 'page': int(page), 'total': count, 'list': finish_list}, None

    @staticmethod
    def search(id_list, need_map=False):
        """
        按照ID搜索信息
        :param id_list: 信息ID列表
        :param need_map: True: {"thead_id":thread_item,...}, False: [thread_item,....]
        :return: data, err
        """
        if not id_list:
            return [], None
        thread_set = Thread.objects.filter(id__in=id_list)
        # 开始按过滤条件
        try:
            thread_set = thread_set \
                .annotate(thread_category_value=F("category_id__value")) \
                .annotate(need_auth=F("category_id__need_auth")) \
                .annotate(thread_classify_value=F("classify_id__value")) \
                .annotate(show_value=F("show_id__value"))

            thread_set = thread_set.filter(is_deleted=0)
            thread_set = thread_set.values(
                'thread_category_value', 'thread_classify_value', "id", "is_deleted", "category_id", "classify_id",
                "show", "user_id", "with_user_id", "title", "subtitle", "content", "summary",
                "access_level", "author", "ip", "has_enroll", "has_fee", "has_comment", "has_location",
                "cover", "photos", "video", "files", "price", "is_original", "link", "create_time",
                "update_time", "logs", "more", "sort", "language_code",
            )
        except Exception as e:
            return None, "err:" + e.__str__()
        thread_set = list(thread_set)

        # 分页数据 主键提取获取 扩展数据(thread_extends)# 扩展字段拼装服务
        category_id_list = list(set([item['category_id'] for item in thread_set if item['category_id']]))
        thread_id_list = list(set([item['id'] for item in thread_set if item['id']]))

        # # 扩展数据拼接
        extend_merge_service = ThreadExtendOutPutService(
            category_id_list=category_id_list,
            thread_id_list=thread_id_list
        )
        finish_set = extend_merge_service.merge(thread_set)

        # 由于有字段冲突，所以这里做一次字段别名处理
        finish_set = filter_result_field(
            result_list=finish_set,
            alias_dict={"price": "thread_price"}
        )
        # 以字典形式返回{"主键"：{数据...}}
        if need_map:
            finish_set = {i['id']: i for i in finish_set}
        return finish_set, None

    @staticmethod
    def search_ids(search_prams=None):
        """
        根据搜索条件查search_prams，询信息表ID
        :param search_prams:
        :return: list
        """
        # 用于条件搜索
        search_prams = format_params_handle(
            param_dict=search_prams,
            filter_filed_list=[
                "title", "subtitle", "access_level", "author",
                "has_enroll", "has_fee", "has_comment", "has_location", "is_original", "finance_invoicing_code",
                "thread_category_value", "thread_classify_value", "platform_code", "need_auth", "show_value"
            ],
            alias_dict={
                "title": "title__contains", "subtitle": "subtitle__contains"
            },
            is_remove_empty=True
        )
        if not search_prams:
            return [], None
        thread_set = Thread.objects
        try:
            thread_set = thread_set \
                .annotate(thread_category_value=F("category__value")) \
                .annotate(platform_code=F("category__platform_code")) \
                .annotate(need_auth=F("category__need_auth")) \
                .annotate(thread_classify_value=F("classify__value")) \
                .annotate(show_value=F("show__value")).filter(is_deleted=0)
            thread_set = thread_set.filter(**search_prams)
            count = thread_set.count()
            # 查询保护，如果筛选条件超出100条，则进行筛选
            if count >= 100:
                return [], None

            thread_set = thread_set.values('id')
        except Exception as e:
            return None, "err:" + e.__str__()
        thread_id_list = [i["id"] for i in list(thread_set)]
        return thread_id_list, None



"""
Created on 2022-04-11
@author:刘飞
@description:发布子模块逻辑处理
"""
import logging

from django.db.models import F

from ..models import ThreadClassify
from ..utils.j_recur import JRecur

log = logging.getLogger()


class ThreadClassifyTreeServices:
    def __init__(self):
        pass

    @staticmethod
    def get_classify_tree(classify_id=None, classify_value=None):
        """
        类别树。
        """
        # 第一步，把类别列表全部读出来
        classify_set = ThreadClassify.objects.annotate(
            classify_value=F('value'),
            category_value=F('category_id__value'),
            show_value=F('show_id__value'),
        ).order_by('sort').values(
            'id',
            'classify_value',
            'name',
            'show_value',
            'category_value',
            'description',
            'icon',
            'sort',
            'parent_id',
        )
        # print("> classify_set:", classify_set)
        classify_list = list(classify_set)
        # print("> classify_list:", classify_list)

        # 第二步，遍历列表，把数据存放在dict里
        filter_key = 'id' if classify_id else ('classify_value' if classify_value else None)
        filter_value = classify_id if classify_id else (classify_value if classify_value else None)
        if filter_key and filter_value:
            classify_tree = JRecur.create_tree(source_list=classify_list, search_root_key=filter_key, search_root_value=filter_value)
            return classify_tree, None
        return None, None

    @staticmethod
    def get_classify_all_tree(classify_value=None, classify_id=None):
        # 第一步，把类别列表全部读出来
        classify_set = ThreadClassify.objects.annotate(classify_value=F('value')).order_by('sort').values(
            'id',
            'classify_value',
            'name',
            'description',
            'icon',
            'sort',
            'parent_id',
        )
        # print("> category_set:", category_set)
        classify_list = list(classify_set)
        # print("> classify_list:", classify_list)

        # 第二步，遍历列表，把数据存放在dict里
        filter_key = 'id' if classify_id else ('classify_value' if classify_value else None)
        filter_value = classify_id if classify_id else (classify_value if classify_value else None)

        classify_tree = JRecur.create_forest(source_list=classify_list)

        if filter_key and filter_value:
            classify_tree = JRecur.filter_forest(classify_tree, filter_key, filter_value)
            if len(classify_tree) == 1:
                classify_tree = classify_tree[0]

        return classify_tree, None



# encoding: utf-8
"""
@project: djangoModel->thread_v2
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis:
@created_time: 2022/7/29 15:11
"""
from django.db.models import F

from xj_user.services.user_detail_info_service import DetailInfoService
from ..models import Thread
from ..models import ThreadExtendData
from ..services.thread_extend_service import ThreadExtendService, ThreadExtendOutPutService
from ..services.thread_statistic_service import StatisticsService
from ..utils.custom_response import util_response
from ..utils.custom_tool import format_params_handle
from ..utils.join_list import JoinList


# 信息服务CURD(支持扩展字段配置)  V2版本
class ThreadItemService:
    @staticmethod
    def add(params):
        # 扩展字段与主表字段拆分
        filter_filed_list = [
            "is_deleted", "category_value", "category_name", "category_id", "classify_id", "classify_value", "classify_name",
            "show_id", "show_value", "show_name", "user_id", "with_user_id",
            "title", "subtitle", "content", "summary", "access_level", "author", "ip", "has_enroll", "has_fee", "has_comment", "has_location",
            "cover", "photos", "video", "files", "price", "is_original", "link",
            "create_time", "update_time", "logs", "more", "sort", "language_code"
        ]
        # 主表 过滤字段
        main_form_data = format_params_handle(
            params.copy(), filter_filed_list=filter_filed_list,
        )
        try:
            instance = Thread.objects.create(**main_form_data)
        except Exception as e:
            return None, f'''{str(e)} in "{str(e.__traceback__.tb_frame.f_globals["__file__"])}" : Line {str(e.__traceback__.tb_lineno)}'''

        # 扩展表 插入或更新
        except_main_form_data = format_params_handle(params.copy(), remove_filed_list=filter_filed_list)
        ThreadExtendService.create_or_update(except_main_form_data, instance.id, main_form_data.get("category_id_id", None))
        return {"id": instance.id}, None

    @staticmethod
    def detail(pk):
        """获取信息内容"""
        thread_dict = Thread.objects.filter(id=pk, is_deleted=False).annotate(
            category_value=F("category__value"),
            category_name=F("category__name"),
            category_platform_code=F("category__platform_code"),
            classify_value=F("classify__value"),
            classify_name=F("classify__name"),
            show_value=F("show__value"),
            show_name=F("show__name"),
        ).values(
            "id", "is_deleted", "category_value", "category_name", "category_id", "classify_id", "classify_value", "classify_name",
            "show_id", "show_value", "show_name", "user_id", "with_user_id",
            "title", "subtitle", "content", "summary", "access_level", "author", "ip", "has_enroll", "has_fee", "has_comment", "has_location",
            "cover", "photos", "video", "files", "price", "is_original", "link",
            "create_time", "update_time", "logs", "more", "sort", "language_code"
        ).first()

        # 信息统计表更新数据
        if not thread_dict:
            return None, "数据不存在"

        # ============ 拼接扩展数据 start ============
        extend_merge_service = ThreadExtendOutPutService(category_id_list=[thread_dict.get("category_id")], thread_id_list=[pk])
        statistic_list = StatisticsService.statistic_list(id_list=[pk])
        user_info_list = DetailInfoService.get_list_detail(user_id_list=[thread_dict["user_id"]] if thread_dict.get("user_id") else [])
        [extend_data_dict] = extend_merge_service.merge([thread_dict])
        [join_statistic_dict] = JoinList(l_list=[extend_data_dict], r_list=statistic_list, l_key="id", r_key='thread_id').join()
        [join_user_info_list] = JoinList(l_list=[join_statistic_dict], r_list=user_info_list, l_key="user_id", r_key='user_id').join()
        # ============ 拼接扩展数据 end  ============
        # 所有访问成功，则进行统计计数
        StatisticsService.increment(thread_id=pk, tag='views', step=1)
        return join_user_info_list, 0

    @staticmethod
    def edit(form_data, pk):
        form_data.setdefault("id", pk)
        # 主表过滤字段
        filter_filed_list = [
            "is_deleted", "category_id", "classify_id", "show", "user_id", "with_user_id", "title", "subtitle",
            "content", "summary", "access_level", "author", "ip", "has_enroll", "has_fee", "has_comment", "has_location",
            "cover", "photos", "video", "files", "price", "is_original", "link", "create_time", "update_time", "logs", "more", "sort",
            "language_code"
        ]
        # 主表修改
        main_res = Thread.objects.filter(id=pk)
        if not main_res:
            return None, "数据不存在，无法进行修改"
        try:
            # 主表修改
            main_form_data = format_params_handle(form_data.copy(), filter_filed_list=filter_filed_list)
            main_res.update(**main_form_data)
            # 扩展字段修改
            # 排除主表之外的字段，理论上就是扩展字段，接下来仅仅需要转换一下扩展字段
            except_main_form_data = format_params_handle(form_data.copy(), remove_filed_list=filter_filed_list)
            return ThreadExtendService.create_or_update(except_main_form_data, pk, main_form_data.get("category_id", None))
        except Exception as e:
            return None, "信息主表写入异常：" + str(e) + "  line:" + str(e.__traceback__.tb_lineno)

    @staticmethod
    def delete(id):
        main_res = Thread.objects.filter(id=id, is_deleted=0)
        if not main_res:
            return None, "数据不存在，无法进行修改"
        main_res.update(is_deleted=1)
        return None, None

    @staticmethod
    def select_extend(id):
        """单独查询 查询扩展字段"""
        return util_response(list(ThreadExtendData.objects.filter(id=id).values()))



"""
Created on 2022-04-11
@description:刘飞
@description:发布子模块逻辑分发
"""
import datetime

from rest_framework.views import APIView

from xj_user.services.user_detail_info_service import DetailInfoService
# from xj_user.services.user_service import UserService
from xj_user.utils.user_wrapper import user_authentication_wrapper
from ..services.thread_list_service import ThreadListService
from ..services.thread_statistic_service import StatisticsService
# from ..utils.custom_authentication_wrapper import authentication_wrapper
from ..utils.custom_response import util_response
from ..utils.custom_tool import request_params_wrapper
from ..utils.join_list import JoinList


class ThreadListAPIView(APIView):
    """
    get: 信息表列表
    post: 信息表新增
    """

    # @authentication_wrapper

    # 我们更希望通过装饰器来做权限验证，这样可以更好的精简API层的代码量 2022.10.3 by Sieyoo
    @user_authentication_wrapper  # 如果有token则返回user_info，无则返回空
    @request_params_wrapper
    def get(self, *args, user_info=None, request_params, **kwargs):
        params = request_params
        filter_fields = request_params.get('filter_fields', None)
        try:
            if params.get('create_time_start'):
                datetime.datetime.strptime(params.get('create_time_start'), "%Y-%m-%d %H:%M:%S")
            if params.get('create_time_end'):
                datetime.datetime.strptime(params.get('create_time_end'), "%Y-%m-%d %H:%M:%S")
        except ValueError:
            return None, f'时间格式错误:它的格式应该是YYYY-MM-DD HH:MM:SS'
        # 弃用，我们更希望通过装饰器来做权限验证，这样可以更好的精简API层的代码量 2022.10.3 by Sieyoo
        # 获取权限,权限验证
        # token = request.META.get('HTTP_AUTHORIZATION', None)
        # print("> ThreadListAPIView token:", token)
        # if token and str(token).strip().upper() != "BEARER":
        #     token_serv, error_text = UserService.check_token(token)
        #     if error_text:
        #         return util_response(err=6000, msg=error_text)
        #     token_serv, error_text = UserService.check_token(token)

        # ==================== 1、先通过用户ID拿到所属权限 ====================
        auth_dict = {}
        user_id = user_info.get("user_id", None) if user_info else None
        # if user_id:
        #     auth_dict, error_text = PermissionService.user_permission_tree(user_id=user_id, module="thread")
        # if error_text:
        #     return util_response(err=1002, msg=error_text)
        # auth_dict = JDict(auth_dict)  # 与当前用户有关的权限值
        # print("> auth_dict", auth_dict)
        # other_role_dict = auth_dict.THREAD and auth_dict.THREAD.OTHER_ROLE_OPERATE or {}  # 他人操作权限
        # self_role_dict = auth_dict.THREAD and auth_dict.THREAD.SELF_ROLE_OPERATE or {}  # 本人操作权限
        # print("> other_role_dict, self_role_dict", other_role_dict, self_role_dict)

        # ==================== 2、再过滤掉不可查看的类别（即同级、子级、父级、外级全禁止的） ====================

        # ==================== 3、再去掉有access_level(访问级别)的信息 ====================

        # ==================== 4、逐个类别查看是否有允许查看和用户ID列表 ====================

        # ban_user_list = []  # 允许读的用户列表
        # allow_user_list = []  # 禁止读的用户列表
        # if auth_dict.GROUP_PARENT and auth_dict.GROUP_PARENT.ban_view.upper() == "Y":
        #     ban_user_list.extend(auth_dict.GROUP_PARENT.user_list)
        # else:
        #     allow_user_list.extend(auth_dict.GROUP_PARENT.user_list if auth_dict.GROUP_PARENT else [])
        #
        # if auth_dict.GROUP_CHILDREN and auth_dict.GROUP_CHILDREN.ban_view.upper() == "Y":
        #     ban_user_list.extend(auth_dict.GROUP_CHILDREN.user_list)
        # else:
        #     allow_user_list.extend(auth_dict.GROUP_CHILDREN.user_list if auth_dict.GROUP_CHILDREN else [])
        #
        # if auth_dict.GROUP_INSIDE and auth_dict.GROUP_INSIDE.ban_view.upper() == "Y":
        #     ban_user_list.extend(auth_dict.GROUP_INSIDE.user_list)
        # else:
        #     allow_user_list.extend(auth_dict.GROUP_INSIDE.user_list if auth_dict.GROUP_INSIDE else [])
        #
        # if not auth_dict.GROUP_ADMINISTRATOR and not auth_dict.GROUP_MANAGER:
        #     if auth_dict.GROUP_OUTSIDE and auth_dict.GROUP_OUTSIDE.ban_view.upper() == "Y":
        #         params['user_id__in'] = allow_user_list
        #     else:
        #         params["user_id__not_in"] = ban_user_list
        # else:
        #     params["is_admin"] = True

        # print("> ThreadListAPIView params:", params)
        # print("> ThreadListAPIView allow_user_list, ban_user_list:", allow_user_list, ban_user_list)

        # 获取列表数据
        thread_serv, error_text = ThreadListService.list(params, filter_fields=filter_fields)
        if error_text:
            return util_response(err=1003, msg=error_text)

        # 按权限自动过滤数据
        # thread_list = AutoPermissionValueService.auto_filter_by_permission(
        #     source_list=thread_serv['list'], user_id=user_id, module='THREAD',
        #     feature_list=['OTHER_ROLE_OPERATE', 'SELF_ROLE_OPERATE', ])
        # print("> thread_list:", len(thread_list), thread_list)
        # thread_serv['list'] = thread_list
        # ID列表拆分
        thread_id_list = list(set([item['id'] for item in thread_serv['list'] if item['id']]))
        user_id_list = list(set([item['user_id'] for item in thread_serv['list'] if item['user_id']]))
        # 用户数据和统计数据
        statistic_list = StatisticsService.statistic_list(id_list=thread_id_list)
        user_info_list = DetailInfoService.get_list_detail(params=None, user_id_list=user_id_list)

        # 根据模块化原则，信息模块是基本模块，但定位模块是扩展模块。不允许信息模块强制捆绑扩展模块。by sieyoo at 20221221
        # location_list, err = LocationService.location_list(params={"thread_id_list": thread_id_list}, need_pagination=False, fields=["thread_id", "name", "region_code", "longitude", "latitude"])
        # if not err:
        #     thread_serv['list'] = JoinList(l_list=thread_serv['list'], r_list=location_list, l_key="id", r_key='thread_id').join()

        # 用户数据(full_name, avatar), 统计数据(statistic),
        thread_serv['list'] = JoinList(l_list=thread_serv['list'], r_list=statistic_list, l_key="id",
                                       r_key='thread_id').join()
        thread_serv['list'] = JoinList(l_list=thread_serv['list'], r_list=user_info_list, l_key="user_id",
                                       r_key='user_id').join()
        # thread_serv['auth_dict'] = auth_dict
        # thread_serv['old_size'] = thread_serv['size']
        # thread_serv['size'] = size
        return util_response(data=thread_serv, is_need_parse_json=True)



"""
Created on 2022-04-11
@author:刘飞
@description:发布子模块逻辑处理
"""
import logging

from django.db.models import F

from xj_user.services.user_platform_service import UserPlatformService
from ..models import ThreadCategory
from ..utils.j_recur import JRecur

log = logging.getLogger()


class ThreadCategoryTreeServices:
    def __init__(self):
        pass

    @staticmethod
    def get_category_tree(category_id=None, category_value=None):
        """
        类别树。
        """

        # 第一步，把类别列表全部读出来
        category_set = ThreadCategory.objects.filter(is_deleted=0).annotate(category_value=F('value')).order_by('sort').values(
            'id',
            'platform_code',
            'category_value',
            'name',
            'need_auth',
            'description',
            'sort',
            'parent_id',
        )
        category_list = list(category_set)

        # 第二步，遍历列表，把数据存放在dict里
        filter_key = 'id' if category_id else ('category_value' if category_value else None)
        filter_value = category_id if category_id else (category_value if category_value else None)

        # 第三步，把所有的数据创建成树
        category_tree = JRecur.create_forest(source_list=category_list)

        # 第四步，查找任意节点下面的数据
        if filter_key and filter_value:
            category_tree = JRecur.filter_forest(category_tree, filter_key, filter_value)
            if len(category_tree) == 1:
                category_tree = category_tree[0]

        return category_tree, None

    @staticmethod
    def get_category_tree_by_user(user_id=None):
        """
        获取这个平台下面的所有分类（结构：类别树）。
        """
        # 第一步，把类别列表全部读出来
        platform_info, err = UserPlatformService.get_platform_info_by_user_id(user_id)
        platform_code_list = [i["platform_code"] for i in platform_info]
        print("platform_info:", platform_info)
        if err:
            return None, err

        category_set = ThreadCategory.objects.filter(
            is_deleted=0,
            platform_code__in=platform_code_list
        ).annotate(category_value=F('value')).order_by('sort').values(
            'id',
            'platform_code',
            'category_value',
            'name',
            'need_auth',
            'description',
            'sort',
            'parent_id',
        )
        category_list = list(category_set)
        # 第二步，遍历列表，把数据存放在dict里
        category_tree = JRecur.create_forest(source_list=category_list)
        # category_tree = JRecur.filter_forest(category_tree, 'platform_code', platform_info.get("platform_code"))
        category_tree = category_tree[0] if len(category_tree) == 1 else category_tree
        return category_tree, None

    @staticmethod
    def get_child_ids(category_id=None, category_value=None):
        if not category_id and not category_value:
            return None, "参数错误"
        category_set = ThreadCategory.objects.filter(is_deleted=0).annotate(category_value=F('value'))
        category_set = category_set.filter(id=category_id) if category_id else category_set.filter(category_value=category_value)
        if not category_set.first():
            return None, "没有找到该类别信息"
        current_category = category_set.values('id', 'parent_id').first()

        category_tree = JRecur.create_forest(source_list=list(ThreadCategory.objects.filter(is_deleted=0).values('id', 'parent_id')))
        filter_category_tree = JRecur.filter_forest(category_tree, "id", current_category["id"])
        out_list = JRecur.filter_tree_values(filter_category_tree, "id", )
        return out_list, None



# encoding: utf-8
"""
@project: djangoModel->extend_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 扩展服务
@created_time: 2022/7/29 15:14
"""
from django.db.models import F

from ..models import ThreadExtendField, Thread, ThreadExtendData


# 服务提供 扩展表的字段映射关系，返回映射后KEY名。
# 达意最好是ThreadExtendInsertService:
class ThreadExtendInputService:
    thread_extend_filed = None

    def __init__(self, form_data):
        """
        :param form_data: 表单
        :param need_all_field: 是否需要全部的扩展字段（查询的时候会用到）
        """
        self.form_data = form_data
        self.form_data['category_id_id'] = self.form_data.pop("category_id", None)
        self.form_data['classify_id_id'] = self.form_data.pop("classify_id", None)
        # 新增或者修改的时候
        self.thread_extend_filed = {}
        category_id = None
        if "id" in self.form_data.keys():  # 修改时候：传了id,没有传classify_id
            category_id = Thread.objects.filter(id=self.form_data.get('id')).first().category_id

        if self.form_data.get('category_id_id', None):
            category_id = self.form_data.get('category_id_id')

        if category_id:
            self.thread_extend_filed = {
                item["field"]: item["field_index"] for item in ThreadExtendField.objects.filter(category_id_id=category_id).values('field', 'field_index')
            }

    # 请求参数转换
    # TODO 弃用 sieyoo
    def transform_param(self):
        # 没有定义扩展映射直接返回，不进行扩展操作
        if self.thread_extend_filed is None:
            return self.form_data, None
        extend_data = {self.thread_extend_filed[k]: self.form_data.pop(k) for k, v in self.form_data.copy().items() if k in self.thread_extend_filed.keys()}
        return self.form_data, extend_data


# 所有的输出信息服务都需要有统一的公共方法
# 暂时定位 output
class ThreadExtendOutPutService():
    # extend_field:扩展数据表的字段如field_1,field_2....
    # field:配置的字段映射
    field_list = None
    extend_field_map = None  # {extend_field:field}
    field_map = None  # {field:extend_field}
    finish_data = None  # 最终完成映射的扩展数据字典

    def __init__(self, category_id_list=None, thread_id_list=None):
        if category_id_list is None:
            raise Exception("category_id_list 必传")
        if thread_id_list is None:
            raise Exception("thread_id_list 必传")
        self.category_id_list = category_id_list
        self.thread_id_list = thread_id_list

        # 字段映射关系
        field_list_set = ThreadExtendField.objects.filter(category_id__in=self.category_id_list)
        self.field_list = []
        if field_list_set:
            self.field_list = list(field_list_set.values())
        self.field_map = {}  # {i["category_id"]: {i["field_index"]: i["field"],....},....}
        for item in self.field_list:
            if self.field_map.get(item["category_id"]):
                self.field_map[item["category_id"]].update({item["field_index"]: item["field"]})
            else:
                self.field_map[item["category_id"]] = {item["field_index"]: item["field"]}

    def out_put(self):
        self.finish_data = {}  # 返回 self.finish_data：{thread_id:{扩展数据},.....} {thread_id:{扩展数据},.....}
        # 获取扩展数据
        extend_data = list(ThreadExtendData.objects.filter(thread_id__in=self.thread_id_list).annotate(category_id=F("thread_id__category_id")).values())
        extend_data = [(i.pop("thread_id_id"), i.pop("category_id"), i) for i in extend_data]
        # 扩展数据 替换KEY
        for thread_id, category_id, item in extend_data:
            category_field = self.field_map.get(category_id, None)
            if category_field is None:
                continue
            remove_none = {k: v for k, v in item.items() if v}
            temp_dict = {}
            for k, v in remove_none.items():
                if category_field.get(k):
                    temp_dict.update({category_field[k]: v})
            self.finish_data[thread_id] = temp_dict
        return self.finish_data

    def merge(self, merge_set, merge_set_key='id'):
        # 把结果集和{thread_id:{扩展数据}}，拼接到 merge_set
        extend_map = self.out_put()
        for item in merge_set:
            if item.get(merge_set_key) and extend_map.get(item[merge_set_key]):
                item.update(extend_map[item[merge_set_key]])
        return merge_set


# 扩展字段增删改查
class ThreadExtendService:
    @staticmethod
    def create_or_update(extend_params, thread_id=None, category_id=None, **kwargs):
        # 不存在扩展字段的时候直接返回成功
        if extend_params is None:
            return None, None
        if thread_id is None:
            return None, "扩展字段修改错误,thread_id不可以为空"
        # 获取扩展字段映射
        thread_obj = Thread.objects.filter(id=thread_id).first()
        if not thread_obj and not category_id:
            return None, None
        category_id = category_id if category_id else thread_obj.category_id
        # 扩展字段映射
        extend_fields = ThreadExtendField.objects.filter(category_id=category_id).values("field_index", "field")
        extend_field_map = {item["field"]: item["field_index"] for item in extend_fields}
        # 扩展数据替换
        transformed_extend_params = {extend_field_map[k]: v for k, v in extend_params.items() if extend_field_map.get(k)}
        # 修改或添加数据
        try:
            extend_obj = ThreadExtendData.objects.filter(thread_id=thread_id)
            if not extend_obj:
                transformed_extend_params.setdefault('thread_id_id', thread_id)
                ThreadExtendData.objects.create(**transformed_extend_params)
                return None, None
            extend_obj.update(**transformed_extend_params)
        except Exception as e:
            return None, "扩展字段修改错误:" + str(e)
        return None, None



# encoding: utf-8
"""
@project: djangoModel->thread_add
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 信息添加接口
@created_time: 2022/8/8 13:36
"""
from rest_framework.views import APIView

from xj_user.services.user_service import UserService
from ..services.thread_item_service import ThreadItemService
from ..utils.custom_response import util_response
from ..utils.custom_tool import parse_data


class ThreadAdd(APIView):
    def post(self, request):
        # token验证
        token = request.META.get('HTTP_AUTHORIZATION', None)
        token_serv, error_text = UserService.check_token(token)
        if error_text:
            return util_response(err=6045, msg=error_text)
        params = parse_data(request)
        # print("ThreadAdd: params:", params)
        # print("ThreadAdd: token_serv:", token_serv)
        params['user_id'] = token_serv.get('user_id', None)
        # # 表单初步验证
        # validator = ThreadAddValidator(params)
        # is_pass, error = validator.validate()
        # if not is_pass:
        #     return util_response(err=4022, msg=error)
        # 插入服务
        data, err_txt = ThreadItemService.add(params)
        if not err_txt:
            return util_response(data=data)
        return util_response(err=47767, msg=err_txt)



"""
Created on 2022-01-19
@auth:刘飞
@description:自定义用户验证
"""

import jwt
from django.conf import settings
from rest_framework import authentication
from rest_framework import exceptions
from xj_user.models import BaseInfo


class Authentication(authentication.BaseAuthentication):
    """用户认证，之后将移植到用户模块"""

    def authenticate(self, request):
        # 验证是否已经登录，函数名必须为：authenticate
        token = request._request.headers.get('Authorization')
        if not token:
            raise exceptions.AuthenticationFailed('用户认证失败！')
        try:
            token = token.split(" ")[-1]
            payload = jwt.decode(token, key=settings.JWT_SECRET_KEY, verify=True, algorithms=["RS256", "HS256"])
            user = BaseInfo.objects.filter(user_name=payload.get('username')).first()
            if not user:
                raise exceptions.AuthenticationFailed('用户认证失败。')
            # 在rest_framework内部会将以下两个元素赋值到request，以供后续使用
            return (user, None)
        except jwt.ExpiredSignatureError:
            raise exceptions.AuthenticationFailed('验证信息已过期，请重新获取！')
        except jwt.InvalidTokenError:
            raise exceptions.AuthenticationFailed('无效令牌！')

    def authenticate_header(self, request):
        # 这个函数可以没有内容，但是必须要有这个函数
        pass



"""
Created on 2022-04-11
@author:刘飞
@description:发布子模块路由分发
"""
from django.urls import re_path

from xj_thread.apis.thread_category_apis import ThreadCategoryApis
from xj_thread.apis.thread_classify_apis import ThreadClassifyApis
from .apis.thread_add import ThreadAdd
from .apis.thread_category_tree import ThreadCategoryTreeAPIView
from .apis.thread_classify_tree import ThreadClassifyTreeAPIView
from .apis.thread_item import ThreadItemAPI
from .apis.thread_list import ThreadListAPIView
from .apis.thread_other_list import AuthListAPIView, ShowListAPIView, TagListAPIView, ThreadExtendFieldList
from .apis.thread_statistic import ThreadStaticAPIView

# 应用名称
# app_name = 'thread'

urlpatterns = [
    re_path(r'^category_add/?$', ThreadCategoryApis.add, name='category_add'),  # 类别列表
    re_path(r'^category_del/?(?P<pk>[-_\w]+)?', ThreadCategoryApis.delete, name='category_del'),  # 类别列表
    re_path(r'^category_edit/?(?P<pk>[-_\w]+)?', ThreadCategoryApis.edit, name='category_edit'),  # 类别列表
    re_path(r'^category_list/?$', ThreadCategoryApis.list, name='category_list'),  # 类别列表
    re_path(r'^category_tree/?(?P<category_value>[-_\w]+)?/?$', ThreadCategoryTreeAPIView.as_view(), name='thread_category_tree'),
    re_path(r'^user_category_tree$', ThreadCategoryTreeAPIView.get_category_tree_by_user, name='thread_category_tree_v2'),

    re_path(r'^classify_list/?$', ThreadClassifyApis.list, name='classify_list'),  # 分类列表
    re_path(r'^classify_add/?$', ThreadClassifyApis.add, name='classify_list'),  # 分类列表
    re_path(r'^classify_edit/?$', ThreadClassifyApis.edit, name='classify_list'),  # 分类列表
    re_path(r'^classify_del/?$', ThreadClassifyApis.edit, name='classify_list'),  # 分类列表
    re_path(r'^classify_tree/?(?P<classify_value>[-_\w]+)?/?$', ThreadClassifyTreeAPIView.as_view(), name='thread_classify_tree'),

    re_path(r'^show_list/?$', ShowListAPIView.as_view(), name='show_list'),  # 展示类型列表
    re_path(r'^show/?(?P<show_value>[-_\w]+)?/?$', ShowListAPIView.as_view(), name='show_list'),  # 展示类型列表
    re_path(r'^tag_list/?$', TagListAPIView.as_view(), name='tag_list'),  # 展示类型列表
    re_path(r'^list/?(?P<category_value>[-_\w]+)?/?$', ThreadListAPIView.as_view(), name='list'),  # 信息列表/新增
    re_path(r'^item_add/?$', ThreadAdd.as_view(), name='list'),  # 信息列表/新增
    re_path(r'^item/(?P<pk>\d+)/?$', ThreadItemAPI.as_view(), name='detail'),  # 信息单挑操作：详情/编辑/删除
    # 列表 信息相关
    re_path(r'^auth[_/]list/?$', AuthListAPIView.as_view(), name='auth_list'),  # 权限列表
    re_path(r'^extend_field_list/?$', ThreadExtendFieldList.as_view(), name='tag_list'),  # 展示类型列表
    re_path(r'^statistic/?$', ThreadStaticAPIView.as_view(), name='tag_list'),  # 计数统计，前端埋点接口
]



"""
Created on 2022-04-11
@description:刘飞
@description:发布子模块逻辑分发
"""
from rest_framework.response import Response
from rest_framework.views import APIView

# from xj_user.utils.custom_authorization import CustomAuthentication
from ..services.thread_classify_tree_service import ThreadClassifyTreeServices
from ..utils.custom_tool import parse_data


# from ..utils.custom_authentication_wrapper import authentication_wrapper


class ThreadClassifyTreeAPIView(APIView):
    """
    get: 信息表列表
    post: 信息表新增
    """

    def get(self, request, classify_value=None, *args, **kwargs):
        params = parse_data(request)
        # print("> ThreadclassifyTreeAPIView params classify_value:", params, classify_value)
        classify_value = classify_value if classify_value else params.get('classify_value', None)
        classify_id = params.get('classify_id', None)
        # print("> ThreadclassifyTreeAPIView classify_value, classify_id:", classify_value, classify_id)
        if classify_value or classify_id:
            classify_serv, error_text = ThreadClassifyTreeServices.get_classify_tree(classify_id=classify_id, classify_value=classify_value)
        else:
            classify_serv, error_text = ThreadClassifyTreeServices.get_classify_all_tree()

        if error_text:
            return Response({'err': 1000, 'msg': error_text})

        return Response({'err': 0, 'msg': 'OK', 'data': classify_serv})



import socket

from django.db import models
from django.utils import timezone
from DjangoUeditor.models import UEditorField

# from apps.user.models import User
# from apps.user.models import BaseInfo

hostname = socket.gethostname()
my_ip_addr = socket.gethostbyname(hostname)


class ThreadShow(models.Model):
    """
    4、Thread_ThreadShow 展示类型表
    展示类型。用于对前端界面的显示样式进行分类
    """

    class Meta:
        db_table = 'thread_show'
        verbose_name_plural = '04. 展示类型表 (样式)'

    id = models.AutoField(verbose_name='展示类型ID', primary_key=True, help_text='')
    value = models.CharField(verbose_name='值', max_length=50, help_text='')
    name = models.CharField(verbose_name='名称', max_length=255, blank=True, null=True, help_text='')
    config = models.JSONField(verbose_name='前端配置', blank=True, null=True, default=list, help_text='')  # 用于存放前端自定义的界面或样式相关的配置数据
    description = models.CharField(verbose_name='描述', max_length=255, blank=True, null=True, help_text='')

    def __str__(self):
        return f"{self.name} ({self.value or ''})"


class ThreadCategory(models.Model):
    """
    2、Thread_ThreadCategory 主类别表
    类别。类似于版块大类的概念，用于圈定信息内容所属的主要类别
    """

    class Meta:
        db_table = 'thread_category'
        verbose_name_plural = '02. 主类别表 (页面类别)'
        # ordering = ['platform_code', 'parent_id__id', 'sort']

    id = models.AutoField(verbose_name='ID', primary_key=True, help_text='')
    platform_code = models.CharField(verbose_name="平台码", max_length=16, blank=True, null=True, default="")
    value = models.CharField(verbose_name='值', max_length=50, help_text='')
    name = models.CharField(verbose_name='名称', max_length=255, blank=True, null=True, help_text='')
    need_auth = models.BooleanField(verbose_name="是否需要权限", blank=True, null=True, help_text='类别是否登录后才能查看')
    description = models.CharField(verbose_name='描述', max_length=255, blank=True, null=True, help_text='')
    sort = models.IntegerField(verbose_name="排序", blank=True, null=True, help_text='默认排序为升序')
    parent = models.ForeignKey(to="self", verbose_name='父类类别', db_column='parent_id', blank=True, null=True, help_text='', on_delete=models.DO_NOTHING)
    is_deleted = models.BooleanField(verbose_name='是否删除', db_column="is_deleted", null=False, blank=True, help_text='')

    def short_description(self):
        if len(str(self.description)) > 30:
            return '{}...'.format(str(self.description)[0:30])
        return str(self.description)

    short_description.short_description = '描述'

    def __str__(self, help_text=''):
        return f"{self.name} ({self.value or ''})"

    # 处理字段出现 前后空格问题
    def clean(self):
        """模型清洗"""
        if self.name:
            self.value = self.value.strip()

    def save(self, *args, **kwargs):
        self.full_clean()
        super(ThreadCategory, self).save(*args, **kwargs)


class ThreadClassify(models.Model):
    """
    3、Thread_ThreadClassify 分类表
    @brief 分类。具体的分类，可以是按行业、兴趣、学科的分类，是主类别下的子分类。
    @note 考虑到多语言翻译的问题，不需要写接口，由运维在后台添加
    """

    class Meta:
        db_table = 'thread_classify'
        verbose_name_plural = '03. 分类表 (行业分类)'
        # ordering = ['category', 'parent_id__id', 'sort']  # TODO 不能在这里写排序，否则会导致所有查询都进行排序，会影响速度，所有已使用代码都要改掉 sieyoo by 20221105

    id = models.AutoField(verbose_name='分类ID', primary_key=True)
    value = models.CharField(verbose_name='分类', max_length=50, unique=True, help_text='')
    name = models.CharField(verbose_name='名称', max_length=255, blank=True, null=True, help_text='')
    show = models.ForeignKey(verbose_name='默认展示ID', to=ThreadShow, db_column='show_id', related_name='+', on_delete=models.DO_NOTHING, null=True, blank=True, help_text='')
    description = models.CharField(verbose_name='描述', max_length=255, blank=True, null=True, help_text='')
    category = models.ForeignKey(verbose_name='所属类别', to=ThreadCategory, db_column='category_id', related_name='+', on_delete=models.DO_NOTHING, blank=True, null=True, help_text='')
    icon = models.CharField(verbose_name='图标', max_length=255, blank=True, null=True, help_text='')
    sort = models.IntegerField(verbose_name="排序", blank=True, null=True, help_text='默认排序为升序')
    parent = models.ForeignKey(to="self", verbose_name='父类分类', db_column='parent_id', blank=True, null=True, help_text='', on_delete=models.DO_NOTHING)
    config = models.JSONField(verbose_name='配置', blank=True, null=True, help_text='')

    def __str__(self, help_text=''):
        return f"{self.name} ({self.value or ''})"


# # 访问权限。作者指定允许哪里用户可以访问，例如私有、公开、好友、指定某些人可以访问等。
# class ThreadAuth(models.Model):
#     class Meta:
#         db_table = 'thread_auth'
#         verbose_name_plural = '权限类型表'
#
#     id = models.AutoField(verbose_name='ID', primary_key=True)
#     value = models.CharField(verbose_name='值', max_length=50)
#
#     def __str__(self):
#         return f"{self.value}"


# 扩展字段数据表。用于扩展一些自定义的版块功能的数据
class ThreadExtendData(models.Model):
    """ 5、Thread_ThreadExtendData 扩展字段数据表 """

    class Meta:
        db_table = 'thread_extend_data'
        verbose_name_plural = '05. 扩展字段数据表'

    # id = models.AutoField(verbose_name='信息ID', primary_key=True, help_text='')
    thread_id = models.OneToOneField(verbose_name='信息ID', to='Thread', related_name="thread_extend_data", db_column='thread_id',
                                     primary_key=True, on_delete=models.DO_NOTHING, help_text='')
    field_1 = models.CharField(verbose_name='自定义字段_1', max_length=255, blank=True, null=True, help_text='')
    field_2 = models.CharField(verbose_name='自定义字段_2', max_length=255, blank=True, null=True, help_text='')
    field_3 = models.CharField(verbose_name='自定义字段_3', max_length=255, blank=True, null=True, help_text='')
    field_4 = models.CharField(verbose_name='自定义字段_4', max_length=255, blank=True, null=True, help_text='')
    field_5 = models.CharField(verbose_name='自定义字段_5', max_length=255, blank=True, null=True, help_text='')
    field_6 = models.CharField(verbose_name='自定义字段_6', max_length=255, blank=True, null=True, help_text='')
    field_7 = models.CharField(verbose_name='自定义字段_7', max_length=255, blank=True, null=True, help_text='')
    field_8 = models.CharField(verbose_name='自定义字段_8', max_length=255, blank=True, null=True, help_text='')
    field_9 = models.CharField(verbose_name='自定义字段_9', max_length=255, blank=True, null=True, help_text='')
    field_10 = models.CharField(verbose_name='自定义字段_10', max_length=255, blank=True, null=True, help_text='')
    field_11 = models.CharField(verbose_name='自定义字段_11', max_length=255, blank=True, null=True, help_text='')
    field_12 = models.CharField(verbose_name='自定义字段_12', max_length=255, blank=True, null=True, help_text='')
    field_13 = models.CharField(verbose_name='自定义字段_13', max_length=255, blank=True, null=True, help_text='')
    field_14 = models.CharField(verbose_name='自定义字段_14', max_length=255, blank=True, null=True, help_text='')
    field_15 = models.CharField(verbose_name='自定义字段_15', max_length=255, blank=True, null=True, help_text='')
    field_16 = models.CharField(verbose_name='自定义字段_16', max_length=255, blank=True, null=True, help_text='')
    field_17 = models.CharField(verbose_name='自定义字段_17', max_length=255, blank=True, null=True, help_text='')
    field_18 = models.CharField(verbose_name='自定义字段_18', max_length=255, blank=True, null=True, help_text='')
    field_19 = models.CharField(verbose_name='自定义字段_19', max_length=255, blank=True, null=True, help_text='')
    field_20 = models.CharField(verbose_name='自定义字段_20', max_length=255, blank=True, null=True, help_text='')

    def __str__(self):
        return f"{self.thread_id}"

    def short_field_1(self):
        if self.field_1 and len(self.field_1) > 25:
            return f"{self.field_1[0:25]}..."
        return self.field_1

    short_field_1.short_description = '自定义字段1'

    def short_field_2(self):
        if self.field_2 and len(self.field_2) > 25:
            return f"{self.field_2[0:25]}..."
        return self.field_2

    short_field_2.short_description = '自定义字段2'

    def short_field_3(self):
        if self.field_3 and len(self.field_3) > 25:
            return f"{self.field_3[0:25]}..."
        return self.field_3

    short_field_3.short_description = '自定义字段3'

    def short_field_4(self):
        if self.field_4 and len(self.field_4) > 25:
            return f"{self.field_4[0:25]}..."
        return self.field_4

    short_field_4.short_description = '自定义字段4'

    def short_field_5(self):
        if self.field_5 and len(self.field_5) > 25:
            return f"{self.field_5[0:25]}..."
        return self.field_5

    short_field_5.short_description = '自定义字段5'

    def short_field_6(self):
        if self.field_6 and len(self.field_6) > 25:
            return f"{self.field_6[0:25]}..."
        return self.field_6

    short_field_6.short_description = '自定义字段6'

    def short_field_7(self):
        if self.field_7 and len(self.field_7) > 25:
            return f"{self.field_7[0:25]}..."
        return self.field_7

    short_field_7.short_description = '自定义字段7'

    def short_field_8(self):
        if self.field_8 and len(self.field_8) > 25:
            return f"{self.field_8[0:25]}..."
        return self.field_8

    short_field_8.short_description = '自定义字段8'

    def short_field_9(self):
        if self.field_9 and len(self.field_9) > 25:
            return f"{self.field_9[0:25]}..."
        return self.field_9

    short_field_9.short_description = '自定义字段9'

    def short_field_10(self):
        if self.field_10 and len(self.field_10) > 25:
            return f"{self.field_10[0:25]}..."
        return self.field_10

    short_field_10.short_description = '自定义字段10'

    def short_field_11(self):
        if self.field_11 and len(self.field_11) > 25:
            return f"{self.field_11[0:25]}..."
        return self.field_11

    short_field_11.short_description = '自定义字段11'

    def short_field_12(self):
        if self.field_12 and len(self.field_12) > 25:
            return f"{self.field_12[0:25]}..."
        return self.field_12

    short_field_12.short_description = '自定义字段12'

    def short_field_13(self):
        if self.field_13 and len(self.field_13) > 25:
            return f"{self.field_13[0:25]}..."
        return self.field_13

    short_field_13.short_description = '自定义字段13'

    def short_field_14(self):
        if self.field_14 and len(self.field_14) > 25:
            return f"{self.field_14[0:25]}..."
        return self.field_14

    short_field_14.short_description = '自定义字段14'

    def short_field_15(self):
        if self.field_15 and len(self.field_15) > 25:
            return f"{self.field_15[0:25]}..."
        return self.field_15

    short_field_15.short_description = '自定义字段15'

    def short_field_16(self):
        if self.field_16 and len(self.field_16) > 25:
            return f"{self.field_16[0:25]}..."
        return self.field_16

    short_field_16.short_description = '自定义字段16'

    def short_field_17(self):
        if self.field_17 and len(self.field_17) > 25:
            return f"{self.field_17[0:25]}..."
        return self.field_17

    short_field_17.short_description = '自定义字段17'

    def short_field_18(self):
        if self.field_18 and len(self.field_18) > 25:
            return f"{self.field_18[0:25]}..."
        return self.field_18

    short_field_18.short_description = '自定义字段18'

    def short_field_19(self):
        if self.field_19 and len(self.field_19) > 25:
            return f"{self.field_19[0:25]}..."
        return self.field_19

    short_field_19.short_description = '自定义字段19'

    def short_field_20(self):
        if self.field_20 and len(self.field_20) > 25:
            return f"{self.field_20[0:25]}..."
        return self.field_20

    short_field_20.short_description = '自定义字段20'


# 扩展字段表。用于声明扩展字段数据表中的(有序)字段具体对应的什么键名。注意：扩展字段是对分类的扩展，而不是主类别的扩展
class ThreadExtendField(models.Model):
    """  6、Thread_ThreadExtendField 扩展字段表 """

    class Meta:
        db_table = 'thread_extend_field'
        verbose_name_plural = '06. 扩展字段表'
        unique_together = (("category_id", "field"),)  # 组合唯一，分类+字段
        # ordering = ['-category_id']

    field_index_choices = [
        ("field_1", "field_1"),
        ("field_2", "field_2"),
        ("field_3", "field_3"),
        ("field_4", "field_4"),
        ("field_5", "field_5"),
        ("field_6", "field_6"),
        ("field_7", "field_7"),
        ("field_8", "field_8"),
        ("field_9", "field_9"),
        ("field_10", "field_10"),
        ("field_11", "field_11"),
        ("field_12", "field_12"),
        ("field_13", "field_13"),
        ("field_14", "field_14"),
        ("field_15", "field_15"),
        ("field_16", "field_16"),
        ("field_17", "field_17"),
        ("field_18", "field_18"),
        ("field_19", "field_19"),
        ("field_20", "field_20"),
    ]
    type_choices = [
        ("string", "string"),
        ("int", "int"),
        ("float", "float"),
        ("bool", "bool"),
        ("select", "select"),
        ("radio", "radio"),
        ("checkbox", "checkbox"),
        ("date", "date",),
        ("time", "time",),
        ("datetime", "datetime"),
        ("moon", "moon"),
        ("year", "year"),
        ("color", "color"),
        ("file", "file"),
        ("image", "image"),
        ("switch", "switch"),
        ("cascader", "cascader"),
        ("plain", "plain"),
        ("textarea", "textarea"),
        ("text", "text"),
        ("number", "number"),
        ("upload", "upload"),
        ("password", "password"),
    ]

    id = models.AutoField(verbose_name='信息ID', primary_key=True, help_text='')
    # 数据库生成classify_id字段
    category = models.ForeignKey(verbose_name='类别ID', null=False, blank=False, to=ThreadCategory,
                                 db_column='category_id', related_name='+', on_delete=models.DO_NOTHING, help_text='')
    field = models.CharField(verbose_name='自定义字段', max_length=255, help_text='')  # 眏射ThreadExtendData表的键名
    field_index = models.CharField(verbose_name='冗余字段', max_length=255, help_text='', choices=field_index_choices)  # 眏射ThreadExtendData表的键名
    value = models.CharField(verbose_name='字段介绍', max_length=255, null=True, blank=True, help_text='')
    type = models.CharField(verbose_name='字段类型', max_length=255, blank=True, null=True, choices=type_choices, help_text='')
    unit = models.CharField(verbose_name='参数单位', max_length=255, blank=True, null=True, help_text='')
    config = models.JSONField(verbose_name='字段配置', blank=True, null=True, default=dict, help_text='')

    def __str__(self):
        return f"{self.id}"


class Thread(models.Model):
    """  1、Thread_Thread 信息主表 """

    class Meta:
        db_table = 'thread'  # 指定数据库的表名，否则默认会显示app名+class名。
        verbose_name_plural = '01. 信息表'  # 指定管理界面的别名，否则默认显示class名。末尾不加s。
        # ordering = ['-create_time']  # TODO 不能在模块设计中写排序，否则会导致所有查询都进行排序，会影响速度，尤其是这张表是大数据表，所有已使用代码都要改掉 sieyoo by 20221105

    bool_choice = [(0, '否'), (1, "是")]
    id = models.BigAutoField(verbose_name='ID', primary_key=True, help_text='')
    is_deleted = models.BooleanField(verbose_name='是否删除', blank=True, null=True, default=0, choices=bool_choice)
    category = models.ForeignKey(verbose_name='类别ID', to=ThreadCategory, on_delete=models.DO_NOTHING, null=True, blank=True, help_text='')
    classify = models.ForeignKey(verbose_name='分类ID', to=ThreadClassify, on_delete=models.DO_NOTHING, null=True, blank=True, help_text='')
    show = models.ForeignKey(verbose_name='展示ID', to=ThreadShow, default=1, null=True, blank=True, on_delete=models.DO_NOTHING, related_name='+', help_text='如果没有传入显示类型，则使用分类表中的默认显示类型')  # 如果没有传入显示类型，则使用分类表中的默认显示类型
    user_id = models.BigIntegerField(verbose_name='用户ID', db_column='user_id', db_index=True, help_text='')
    with_user_id = models.BigIntegerField(verbose_name='与用户ID', db_column='with_user_id', blank=True, null=True, db_index=True, help_text='')
    title = models.CharField(verbose_name='标题', max_length=255, blank=True, null=True, db_index=True, help_text='')
    subtitle = models.CharField(verbose_name='子标题', max_length=255, blank=True, null=True, db_index=True, help_text='')
    content = UEditorField(verbose_name='内容', blank=True, null=True, help_text='信息列表页是不返回内容字段的，因为这会增加数据的体积')
    summary = models.CharField(verbose_name='摘要', max_length=1024, blank=True, null=True, default="", help_text='')
    access_level = models.IntegerField(verbose_name='访问级别', blank=True, null=True, help_text='')  # add-2022-05-20
    author = models.CharField(verbose_name='作者', max_length=255, blank=True, null=True, help_text='')  # add-2022-05-20
    ip = models.GenericIPAddressField(verbose_name='IP地址', blank=True, null=True, protocol='both', default=socket.gethostbyname(socket.gethostname()))  # 只记录创建时的IP
    has_enroll = models.BooleanField(verbose_name='有报名', blank=True, null=True, help_text='')
    has_fee = models.BooleanField(verbose_name='有小费', blank=True, null=True, help_text='')
    has_comment = models.BooleanField(verbose_name='有评论', blank=True, null=True, help_text='')
    has_location = models.BooleanField(verbose_name='有定位', blank=True, null=True, help_text='')
    cover = models.CharField(verbose_name='封面', max_length=1024, blank=True, null=True, help_text='')
    photos = models.JSONField(verbose_name='照片集', blank=True, null=True, help_text='')  # 对象数组，存放{id, url} 获取列表时使用，查看详细时再匹配资源表
    video = models.CharField(verbose_name='视频', max_length=1024, blank=True, null=True, help_text='')
    files = models.JSONField(verbose_name='文件集', blank=True, null=True, help_text='')  # 对象数组，存放{id, url}
    price = models.DecimalField(verbose_name='价格', max_digits=32, decimal_places=8, db_index=True, null=True, blank=True, help_text='')  # add-2022-05-20
    is_original = models.BooleanField(verbose_name='是否原创', blank=True, null=True, help_text='')  # add-2022-05-20
    link = models.CharField(verbose_name='参考链接', max_length=1024, blank=True, null=True, help_text='跳转/参考链接')
    create_time = models.DateTimeField(verbose_name='创建时间', blank=True, null=True, default=timezone.now, help_text='')
    update_time = models.DateTimeField(verbose_name='更新时间', blank=True, null=True, auto_now=True, help_text='')  # 不显示，系统自动填。
    logs = models.JSONField(verbose_name='日志', blank=True, null=True, default=list, help_text='')  # 用户的修改记录等日志信息，数组对象类型 使用CRC32来比较哪些字段被修改过，并记录
    more = models.JSONField(verbose_name='更多信息', blank=True, null=True, help_text='')
    sort = models.BigIntegerField(verbose_name="排序", blank=True, null=True, help_text='默认排序为升序')
    language_code = models.CharField(verbose_name='语言代码', max_length=32, blank=True, null=True, help_text='')

    def __str__(self):
        if len(str(self.title)) > 30:
            return f"({self.id}) {str(self.title)[0:30]}..."
        return f"({self.id}) {self.title}"

    # 判断指定字段长度,超出部分用省略号代替
    def short_title(self):
        if len(str(self.title)) > 30:
            return f'{str(self.title)[0:30]}...'
        return str(self.title)

    short_title.short_description = '标题'

    # 判断指定字段长度,超出部分用省略号代替
    def short_subtitle(self):
        if len(str(self.subtitle)) > 30:
            return f'{str(self.subtitle)[0:30]}...'
        return str(self.subtitle)

    short_subtitle.short_description = '子标题'

    # 判断指定字段长度,超出部分用省略号代替
    def short_summary(self):
        if len(str(self.summary)) > 30:
            return f'{str(self.summary)[0:30]}...'
        return str(self.summary)

    short_summary.short_description = '摘要'

    # 判断指定字段长度,超出部分用省略号代替
    def short_content(self):
        if len(str(self.content)) > 30:
            return '{}...'.format(str(self.content)[0:30])
        return str(self.content)

    # 字段数据处理后,字段verbose_name参数失效
    # 需要重新指定,否则列表页字段名显示的是方法名(short_content)
    short_content.short_description = '内容'

    def short_cover(self):
        if len(str(self.cover)) > 15:
            return '{}...'.format(str(self.cover)[0:15])
        return str(self.cover)

    short_cover.short_description = '封面'

    def short_video(self):
        if len(str(self.video)) > 15:
            return '{}...'.format(str(self.video)[0:15])
        return str(self.video)

    short_video.short_description = '视频'

    def short_photos(self):
        if len(str(self.photos)) > 15:
            return '{}...'.format(str(self.photos)[0:15])
        return str(self.photos)

    short_photos.short_description = '照片集'

    def short_files(self):
        if len(str(self.files)) > 15:
            return '{}...'.format(str(self.files)[0:15])
        return str(self.files)

    short_files.short_description = '文件集'

    def short_logs(self):
        if len(str(self.logs)) > 15:
            return '{}...'.format(str(self.logs)[0:15])
        return str(self.logs)

    short_logs.short_description = '日志'

    def short_more(self):
        if len(str(self.more)) > 30:
            return '{}...'.format(str(self.more)[0:30])
        return str(self.logs)

    short_more.short_description = '更多信息'


class ThreadStatistic(models.Model):
    """ 10、Thread_ThreadStatistic 信息统计表 """
    thread_id = models.BigIntegerField(verbose_name='信息主表', primary_key=True, db_column="thread_id", help_text='')
    flag_classifies = models.CharField(verbose_name='分类标识', max_length=255, null=True, blank=True, help_text='')
    flag_weights = models.CharField(verbose_name='权重标识', max_length=255, null=True, blank=True, help_text='')
    weight = models.FloatField(verbose_name='权重', default=0, db_index=True, help_text='')
    views = models.IntegerField(verbose_name='浏览数', default=0, help_text='')
    plays = models.IntegerField(verbose_name='完阅数', default=0, help_text='')
    comments = models.IntegerField(verbose_name='评论数', default=0, help_text='')
    likes = models.IntegerField(verbose_name='点赞数', default=0, help_text='')
    favorite = models.IntegerField(verbose_name='收藏数', default=0, help_text='')
    shares = models.IntegerField(verbose_name='分享数', default=0, help_text='')

    class Meta:
        db_table = 'thread_statistic'
        verbose_name = '10. 信息统计表'
        verbose_name_plural = verbose_name


class ThreadTag(models.Model):
    """
    7、Thread_ThreadTag 标签类型表
    标签类型，存放预置标签。用于智能化推送信息，以及关键字检索。未来应设计成根据信息内容自动生成标签。
    """

    class Meta:
        db_table = 'thread_tag'
        verbose_name_plural = '07. 标签类型表'

    id = models.AutoField(verbose_name='ID', primary_key=True, help_text='')
    value = models.CharField(verbose_name='标签名', max_length=255, blank=True, null=True, help_text='')
    thread = models.ManyToManyField(to='Thread', through='ThreadTagMapping', through_fields=('tag_id', 'thread_id'), blank=True, help_text="")

    def __str__(self):
        return f"{self.value}"


class ThreadTagMapping(models.Model):
    """
    8、Thread_ThreadTagMapping 标签映射表
    标签映射，存放数据。即将标签和信息关联起来 """

    class Meta:
        db_table = 'thread_tag_mapping'
        verbose_name_plural = '08. 标签映射表'

    id = models.AutoField(verbose_name='ID', primary_key=True, help_text='')
    thread_id = models.ForeignKey(verbose_name='信息ID', to=Thread, db_column='thread_id', related_name='+', on_delete=models.DO_NOTHING, help_text='')
    tag_id = models.ForeignKey(verbose_name='标签ID', to=ThreadTag, db_column='tag_id', related_name='+', on_delete=models.DO_NOTHING, help_text='')

    def __str__(self):
        return f"{self.id}"


class ThreadImageAuth(models.Model):
    """
    9、Thread_ThreadTagMapping 图片权限表
    图片权限。作者可以指定上传的图片的访问权限。如公开照片、阅后即焚、已焚毁、红包、红包阅后即焚、红包阅后已焚毁
    """

    class Meta:
        db_table = 'thread_image_auth'
        verbose_name_plural = '09. 图片权限表'

    id = models.AutoField(verbose_name='ID', primary_key=True, help_text='')
    value = models.CharField(verbose_name='值', max_length=255, blank=True, null=True, help_text='')

# # 图片信息表。用于存放图片的各种信息，存放图片地址但不存放图。
# class ThreadResource(models.Model):
#     class Meta:
#         db_table = 'thread_resource'
#         verbose_name_plural = '图片表'
#
#     id = models.AutoField(verbose_name='ID', primary_key=True, help_text='')
#     name = models.CharField(verbose_name='图片名称', max_length=255, null=True, blank=True, help_text='')
#     url = models.CharField(verbose_name='图片链接', max_length=1024, null=True, blank=True, help_text='')
#     filename = models.CharField(verbose_name='文件名', max_length=255, null=True, blank=True, help_text='')
#     filetype = models.SmallIntegerField(verbose_name='文件类型', null=True, blank=True, help_text='')  # 文件类型0:图片，1:视频，2:文件
#     format = models.CharField(verbose_name='文件格式', max_length=50, help_text='')
#     image_auth_id = models.ForeignKey(verbose_name='图片权限ID', to=ThreadImageAuth, db_column='image_auth_id', related_name='+', on_delete=models.DO_NOTHING, null=True, blank=True, help_text='')
#     price = models.DecimalField(verbose_name='价格', max_digits=32, decimal_places=8, db_index=True, null=True, blank=True, help_text='')
#     snapshot = models.JSONField(verbose_name='快照', blank=True, null=True, help_text='')  # 存放图片的快照数据，如缩略图等。json对象
#     logs = models.JSONField(verbose_name='日志', blank=True, null=True, help_text='')  # 用于存放点击量，点赞量等,数组对象
#     # user_id = models.ForeignKey(verbose_name='用户ID', to=User, db_column='user_id', related_name='+', on_delete=models.DO_NOTHING)
#     user_id = models.BigIntegerField(verbose_name='用户ID', help_text='')
#     thread = models.ManyToManyField(to='Thread', through='ThreadToResource', through_fields=('resource_id', 'thread_id'), blank=True, help_text='')


# # 标签映射，存放数据。即将标签和信息关联起来
# class ThreadToResource(models.Model):
#     class Meta:
#         db_table = 'thread_to_resource'
#         verbose_name_plural = '图文关联表'
#
#     id = models.AutoField(verbose_name='ID', primary_key=True, help_text='')
#     thread_id = models.ForeignKey(verbose_name='信息ID', to=Thread, db_column='thread_id', related_name='+', on_delete=models.DO_NOTHING, help_text='')
#     resource_id = models.ForeignKey(verbose_name='图片ID', to=ThreadResource, db_column='resource_id', related_name='+', on_delete=models.DO_NOTHING, help_text='')



"""
Created on 2022-04-11
@description:刘飞
@description:发布子模块逻辑分发
"""
from rest_framework.views import APIView

from ..services.thread_other_list_service import ThreadOtherListServices
from ..utils.custom_response import util_response
from ..utils.custom_tool import parse_data, request_params_wrapper

t = ThreadOtherListServices()


class CategoryListAPIView(APIView):
    """
    get:类别列表
    """

    @request_params_wrapper
    def get(self, request=None, request_params=None, *args, **kwargs):
        data, error_text = t.thread_category(request_params)
        return util_response(data=data)


class ClassifyListAPIView(APIView):
    """
    get:分类列表
    """

    def get(self, request, *args, **kwargs):
        request_params = parse_data(request)
        category_id = request_params.get('category_id', None)
        category_value = request_params.get('category_value', None)
        classify_id = request_params.get('classify_id', None)
        classify_value = request_params.get('classify_value', None)
        data, error_text = t.thread_classify(category_id=category_id, category_value=category_value, classify_id=classify_id, classify_value=classify_value)
        return util_response(data=data)


class ShowListAPIView(APIView):
    """
    get:展示类型列表
    """

    @request_params_wrapper
    def get(self, *args, request_params=None, **kwargs):
        if request_params is None:
            request_params = {}

        try:
            need_pagination = int(request_params.get("need_pagination", 0))
        except ValueError:
            need_pagination = 0

        data, error_text = t.thread_show(request_params, need_pagination)
        if error_text:
            return util_response(err=1000, msg=error_text)
        return util_response(data=data)


class AuthListAPIView(APIView):
    """
    get:访问权限列表
    """

    def get(self, request, *args, **kwargs):
        data, error_text = t.thread_auth()
        return util_response(data=data)


class TagListAPIView(APIView):
    """
    get:标签列表
    """

    def get(self, request):
        params = parse_data(request)
        data, error_text = t.thread_tag(params)
        return util_response(data=data)


class ThreadExtendFieldList(APIView):
    """扩展字段列表"""

    def get(self, request):
        params = parse_data(request)
        data, err = t.thread_extend_field_list(params)
        if err:
            return util_response(err=54555, msg=err)
        return util_response(data=data)
